@@ -3,12 +3,12 @@
 %%
 %%     Functions for reading and writing TIF files.
 %%
-%%  Copyright (c) 2001 Dan Gudmundsson
+%%  Copyright (c) 2001-2007 Dan Gudmundsson
 %%
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: e3d__tif.erl,v 1.14 2004/10/07 08:32:00 dgud Exp $
+%%     $Id: e3d__tif.erl 206 2008-01-04 16:08:58Z bjorng $
 %%
 
 -module(e3d__tif).
@@ -314,7 +314,7 @@ get_info([{?BitsPerSample, Type = short, Count, {offset, Off}}|R], Tif, Orig,Enc
 % 	    Bps = [8,8,8|_] -> Bps;
 % 	    Err ->
 % 		io:format("~p: Unsupported BitsPerSample ~p ~n", [?MODULE, Err]),
-% 		erlang:fault({?MODULE, unsupported, bitsPerSample})
+% 		erlang:error({?MODULE, unsupported, bitsPerSample})
 % 	end,
     get_info(R, Tif#tif{bps = Bpp}, Orig,Enc);
 get_info([{?SamplesPerPixel, _, 1, {value, SPP}}|R], Tif, Orig,Enc) ->
@@ -597,7 +597,7 @@ lzw_decomp(S, Read, PrevCode, Count, BitLen, Acc) ->
 		{?LZW_EOI, _} ->
 %%		    io:format("~nEOI-2 ~p ~n", [{S, PrevCode, Count, BitLen}]),
 		    Acc;
-		{NewCode, NS2} when integer(NewCode) -> 
+		{NewCode, NS2} when is_integer(NewCode) -> 
 		    Str = ?get_lzw(NewCode),
 		    lzw_decomp(NS2, Read, NewCode, 258, 9, [Str|Acc]);
 	    	Else ->
@@ -605,7 +605,7 @@ lzw_decomp(S, Read, PrevCode, Count, BitLen, Acc) ->
 			      [?MODULE, Else, {NS, PrevCode, Count, BitLen}]),
 		    exit({badly_compressed_data})
 	    end;
-	{NewCode, NS} when integer(NewCode) ->
+	{NewCode, NS} when is_integer(NewCode) ->
 	    case ?get_lzw(NewCode) of
 		undefined when Count == NewCode ->
 		    OldStr = [H|_] = ?get_lzw(PrevCode),
@@ -671,11 +671,11 @@ lzw_compress(<<>>, CC, _W, Omega, BitLen, _TabCount, Build, Acc) ->
     {{TotBitLen, Codes}, N2acc} = lzw_write({NewBL,?LZW_EOI}, NBuild, Nacc),
     PaddL = 8 - (TotBitLen rem 8),
     case catch lzw_buildbin(lists:reverse([{PaddL, 0}|Codes])) of
-        Bin when binary(Bin) -> 
+        Bin when is_binary(Bin) -> 
             list_to_binary(lists:reverse([Bin|N2acc]));
 	_Else ->
 	    io:format("~p:~p Error ~p ~p ~n", [?MODULE, ?LINE, {PaddL, Codes}, CC]),
-	    erlang:fault({?MODULE, decoder, {internal_error, ?LINE}})
+	    erlang:error({?MODULE, decoder, {internal_error, ?LINE}})
     end;
 lzw_compress(Bin, CC, W, Omega, BitLen, TabCount, Build, Acc) when CC == W ->
     Code =?get_lzw(Omega),
@@ -703,31 +703,31 @@ lzw_compress(<<Char:8, Bin/binary>>, CC, W, Omega, BitLen, TabC, Build, Acc) ->
     end.
 
 lzw_write({_,undefined}, _, _) -> 
-    erlang:fault({undef,value});
+    erlang:error({undef,value});
 lzw_write({CLen, Code}, {Totlen, List}, Acc) ->
     NewLen = CLen + Totlen,
     if 
 	NewLen rem 8 == 0 ->
 	    case catch lzw_buildbin(lists:reverse([{CLen,Code}|List])) of
-		Bin when binary(Bin) ->
+		Bin when is_binary(Bin) ->
 		    {{0, []}, [Bin|Acc]};
-		{Bin, NewList} when binary(Bin) ->
+		{Bin, NewList} when is_binary(Bin) ->
 		    Sum = lists:foldl(fun({X,_}, Sum) -> X + Sum end, 0, NewList),
 		    {{Sum, lists:reverse(NewList)}, [Bin|Acc]};
 		Else ->
 		    io:format("~p:~p Error ~p ~p ~n", [?MODULE, ?LINE, Else, 
 						       [{CLen, Code}, {Totlen, List}]]),
-		    erlang:fault({?MODULE, decoder, {internal_error, ?LINE}})
+		    erlang:error({?MODULE, decoder, {internal_error, ?LINE}})
 	    end;
 	NewLen > 100 -> 
 	    case catch lzw_buildbin(lists:reverse([{CLen,Code}|List])) of
-		{Bin, NewList} when binary(Bin) ->		    
+		{Bin, NewList} when is_binary(Bin) ->		    
 		    Sum = lists:foldl(fun({X,_}, Sum) -> X + Sum end, 0, NewList),
 		    {{Sum, lists:reverse(NewList)}, [Bin|Acc]};
 		Else ->
 		    io:format("~p:~p Error ~p ~p ~n", [?MODULE, ?LINE, Else, 
 						       [{CLen, Code}, {Totlen, List}]]),
-		    erlang:fault({?MODULE, decoder, {internal_error, ?LINE}})
+		    erlang:error({?MODULE, decoder, {internal_error, ?LINE}})
 	    end;
 	true ->
 	    {{Totlen + CLen,[{CLen,Code}|List]}, Acc}
