@@ -3,12 +3,12 @@
 %%
 %%     Handle images (2D) and different file formats.
 %%
-%%  Copyright (c) 2001-2004 Dan Gudmundsson
+%%  Copyright (c) 2001-2008 Dan Gudmundsson
 %%
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: e3d_image.erl,v 1.18 2005/06/20 20:22:01 dgud Exp $
+%%     $Id: e3d_image.erl 206 2008-01-04 16:08:58Z bjorng $
 %%
 
 -module(e3d_image).
@@ -43,7 +43,7 @@
 %%       Options {type, Type} and/or {alignment, N} and/or {order, O} see e3d_image.hrl
 load(FileName) ->
     load(FileName, []).
-load(FileName, Opts) when list(FileName), list(Opts) ->
+load(FileName, Opts) when is_list(FileName), is_list(Opts) ->
     Extension = file_extension(FileName),
     Res = case ext_to_type(Extension) of
 	      tga -> e3d__tga:load(FileName, Opts);
@@ -52,7 +52,7 @@ load(FileName, Opts) when list(FileName), list(Opts) ->
 	      png -> e3d__png:load(FileName, Opts);
 	      _ -> return_error({not_supported,Extension})
 	  end,
-    fix_outtype(Res, Opts).
+    fix_outtype(FileName, Res, Opts).
 
 %% Func: save(#e3d_image, Filename [, Opts])
 %% Rets: ok | {error, Reason}
@@ -99,10 +99,10 @@ format_error(Other) ->
 %% Func: convert(#e3d_image, NewType [,NewAlignment [,NewOrder ]])
 %% Rets: #e3d_image | {error, Reason}
 %% Desc: Converts an image to new type optionally NewAlignment and NewOrder
-convert(In, ToType) when atom(ToType) ->
+convert(In, ToType) when is_atom(ToType) ->
     convert(In, ToType, In#e3d_image.alignment, In#e3d_image.order).
 
-convert(In, ToType, NewAlignment) when atom(ToType) ->
+convert(In, ToType, NewAlignment) when is_atom(ToType) ->
     convert(In, ToType, NewAlignment, In#e3d_image.order).
 
 convert(#e3d_image{type=Type, alignment=Al,order=O}=In, Type, Al, O) -> In;
@@ -337,7 +337,7 @@ lowercase([H|R]) ->
 lowercase([]) ->
     [].
 
-fix_outtype(Res = #e3d_image{}, Opts) ->
+fix_outtype(File, Res = #e3d_image{}, Opts) ->
     Type = 
 	case lists:keysearch(type, 1, Opts) of
 	    {value, {type, T}} -> T;
@@ -353,8 +353,8 @@ fix_outtype(Res = #e3d_image{}, Opts) ->
 	    {value, {order, O}} -> O;
 	    false -> Res#e3d_image.order
 	end,
-    convert(Res, Type, Alignment, Order);
-fix_outtype(Res, _) ->  %% Propagate Error Case
+    convert(Res#e3d_image{filename=File}, Type, Alignment, Order);
+fix_outtype(_, Res, _) ->  %% Propagate Error Case
     Res.
 
 
