@@ -4,12 +4,12 @@
 %%     Functions for reading and writing 3D Studio Max files (.tds),
 %%     version 3.
 %%
-%%  Copyright (c) 2001-2005 Bjorn Gustavsson
+%%  Copyright (c) 2001-2007 Bjorn Gustavsson
 %%
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: e3d_tds.erl,v 1.45 2005/03/13 16:38:10 bjorng Exp $
+%%     $Id: e3d_tds.erl 206 2008-01-04 16:08:58Z bjorng $
 %%
 
 -module(e3d_tds).
@@ -702,7 +702,7 @@ make_percent(Percent0) when is_float(Percent0) ->
 make_chunk(Tag, Contents) when is_binary(Contents) ->
     Size = size(Contents) + 6,
     [<<Tag:16/little,Size:32/little>>|Contents];
-make_chunk(Tag, Contents) when list(Contents) ->
+make_chunk(Tag, Contents) when is_list(Contents) ->
     make_chunk(Tag, list_to_binary(Contents)).
 
 %%%
