@@ -3,13 +3,13 @@
 %%
 %%     The UV parametrisation algorithms.
 %%
-%%  Copyright (c) 2002-2004 Dan Gudmundsson, Raimo Niskanen,
+%%  Copyright (c) 2002-2007 Dan Gudmundsson, Raimo Niskanen,
 %%                     Bjorn Gustavsson
 %%
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: auv_mapping.erl,v 1.82 2006/04/26 12:25:41 dgud Exp $
+%%     $Id: auv_mapping.erl 210 2008-01-05 17:43:04Z bjorng $
 %%
 
 %%%%%% Least Square Conformal Maps %%%%%%%%%%%%
@@ -128,7 +128,6 @@ clamp(X) when X < -1.0 -> -1.0;
 clamp(X) -> X.
 
 fix_positions(_V,{_,_,Z},Proj,_) when Z > 0.0 -> Proj;
-fix_positions(_V,_,Proj,undefined) -> Proj;
 fix_positions(V,_,Proj = {X,Y,Z},Tags) ->
     case gb_sets:is_member(V,Tags) of
 	true when X > 0.0 -> 
@@ -139,7 +138,6 @@ fix_positions(V,_,Proj = {X,Y,Z},Tags) ->
 	    Proj
     end.
 
-leftOrRight(undefined, _, _We) -> undefined;
 leftOrRight({LL,LR}, Free0, We) ->
     Del = fun(#be{face=F},{Fs,Ch}) -> {[F|Fs],gb_sets:delete_any(F,Ch)} end,
     {F1,Free1} = foldl(Del,{[],gb_sets:from_list(Free0)},LL),
@@ -702,9 +700,9 @@ project_tri(P0,P1,P2) ->
      e3d_vec:len(L),0.0,
      e3d_vec:dot(T,X), e3d_vec:dot(T,Y)}.
     
-lsq(L, Lpuv) when list(Lpuv) ->
+lsq(L, Lpuv) when is_list(Lpuv) ->
     lsq(L, Lpuv, env);
-lsq(Name, Method) when atom(Method) ->
+lsq(Name, Method) when is_atom(Method) ->
     {ok, [{L, Lpuv}]} = file:consult(Name),
     lsq(L, Lpuv, Method).
 
@@ -942,16 +940,15 @@ lsq_result(X, Lquv, Rdict) ->
 %%
 %% Return: {L_remaining, L_picked}
 %%
-pick(L, P) 
-  when list(L), list(P) ->
+pick(L, P) when is_list(L), is_list(P) ->
     case pick(1, L, P, [], []) of
 	{_, _} = Ok ->
 	    Ok;
 	Fault ->
-	    erlang:fault(Fault, [L, P])
+	    erlang:error(Fault, [L, P])
     end;
 pick(L, P) ->
-    erlang:fault(badarg, [L, P]).
+    erlang:error(badarg, [L, P]).
 
 pick(_, L, [], R, Q) ->
     {lists:reverse(R, L), lists:reverse(Q)};
@@ -974,16 +971,15 @@ pick(_, _, _, _, _) ->
 %% S: List of {Pos,Term} tuples with Term to be 
 %%    inserted at position Pos in L
 %%
-insert(L, S)
-  when list(L), list(S) ->
+insert(L, S) when is_list(L), is_list(S) ->
     case insert(1, L, S, []) of
-	R when list(R) ->
+	R when is_list(R) ->
 	    R;
 	Fault ->
-	    erlang:fault(Fault, [L, S])
+	    erlang:error(Fault, [L, S])
     end;
 insert(L, S) ->
-    erlang:fault(badarg, [L, S]).
+    erlang:error(badarg, [L, S]).
 
 insert(_, L, [], R) ->
     lists:reverse(R, L);
