@@ -8,7 +8,7 @@
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: auv_seg_ui.erl,v 1.43 2006/03/14 13:52:20 dgud Exp $
+%%     $Id: auv_seg_ui.erl 210 2008-01-05 17:43:04Z bjorng $
 %%
 
 -module(auv_seg_ui).
@@ -223,7 +223,7 @@ seg_event_6({action,{material,Cmd}}, #seg{st=St0}=Ss) ->
     end;
 seg_event_6({action,{auv_segmentation,Cmd}}, Ss) ->
     seg_command(Cmd, Ss);
-seg_event_6({callback, Fun}, _) when function(Fun) ->
+seg_event_6({callback, Fun}, _) when is_function(Fun, 0) ->
     Fun();
 seg_event_6({message,Message}, _) ->
     wings_u:message(Message);
