@@ -3,12 +3,12 @@
 %%
 %%     FBX file import/export.
 %%
-%%  Copyright (c) 2003-2006 Bjorn Gustavsson
+%%  Copyright (c) 2003-2007 Bjorn Gustavsson
 %%
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: wpc_fbx.erl,v 1.10 2006/01/20 14:47:02 giniu Exp $
+%%     $Id: wpc_fbx.erl 177 2007-12-21 18:56:58Z bjorng $
 %%
 
 -module(wpc_fbx).
@@ -76,10 +76,10 @@ command(_, _) ->
     next.
 
 menu_entry(Menu) ->
-    Menu ++ [{"Alias FBX (.fbx)...",fbx,[option]}].
+    Menu ++ [{"Autodesk FBX (.fbx)...",fbx,[option]}].
 
 props() ->
-    [{ext,".fbx"},{ext_desc,?__(1,"Alias FBX file")}].
+    [{ext,".fbx"},{ext_desc,?__(1,"Autodesk FBX file")}].
 
 %%%
 %%% Export.
