@@ -3,12 +3,12 @@
 %%
 %%     Toxic Plugin User Interface and Exporter.
 %%
-%%  Copyright (c) 2004 Dan Gudmundsson, Raimo Niskanen
+%%  Copyright (c) 2004-2008 Dan Gudmundsson, Raimo Niskanen
 %%
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: wpc_toxic.erl,v 1.21 2006/01/08 18:02:00 giniu Exp $
+%%     $Id: wpc_toxic.erl 218 2008-01-07 19:53:20Z bjorng $
 %%
 
 -module(wpc_toxic).
@@ -168,9 +168,13 @@ init_pref() ->
     ok.
 
 maybe_append(Condition, Menu, PluginMenu) ->
-    case is_plugin_active(Condition) of
-	false ->    Menu;
-	_ ->     Menu++PluginMenu
+    case {is_plugin_active(Condition),Menu} of
+	{_,[plugin_manager_category]} ->
+	    Menu++PluginMenu;
+	{false,_} ->
+	    Menu;
+	{_,_} ->
+	    Menu++PluginMenu
     end.
 
 is_plugin_active(Condition) ->
@@ -322,7 +326,7 @@ light_dialog(_Name, Ps) ->
 	      [{title,?__(5,"Toxic Options")},{key,{?TAG,minimized}},{minimized,Minimized}]}]
     end.
 
-max_hook(Key, Values) when list(Values) ->
+max_hook(Key, Values) when is_list(Values) ->
     {hook,
      fun (is_minimized, {_Var,I,Sto}) when is_integer(Key) ->
 	     not lists:member(gb_trees:get(I+Key, Sto), Values);
@@ -794,15 +798,15 @@ fixName2([A|R]) ->
     [A|fixName2(R)];
 fixName2([]) -> [].
 
-mkName(Name, [Mat0]) when list(Name) ->
+mkName(Name, [Mat0]) when is_list(Name) ->
     Mat = to_list(Mat0),
     fixName(Name) ++ "_" ++ fixName(Mat);
 mkName(Name, Mat) ->
-    erlang:fault({strange_names, Name,Mat}).
+    erlang:error({strange_names, Name,Mat}).
 
-to_list(Mat) when atom(Mat) ->
+to_list(Mat) when is_atom(Mat) ->
     atom_to_list(Mat);
-to_list(Mat) when list(Mat) ->
+to_list(Mat) when is_list(Mat) ->
     Mat;
 to_list(Mat) ->
     exit({strange_matname, Mat}).
@@ -1198,7 +1202,7 @@ open(Filename, export) ->
 	{ok, F} ->
 	    F;
 	Error ->
-	    erlang:fault(Error, [Filename, export])
+	    erlang:error(Error, [Filename, export])
     end.
 
 %% println(F) ->
@@ -1209,31 +1213,31 @@ open(Filename, export) ->
 % 	ok ->
 % 	    ok;
 % 	Error ->
-% 	    erlang:fault(Error, [F,DeepString])
+% 	    erlang:error(Error, [F,DeepString])
 %     end.
 
 println(F, DeepString) ->
     case file:write(F, [DeepString,io_lib:nl()]) of
 	ok ->    ok;
-	Error -> erlang:fault(Error, [F,DeepString])
+	Error -> erlang:error(Error, [F,DeepString])
     end.
 
 print(F, Format, Args) ->
     case file:write(F, io_lib:format(Format, Args)) of
  	ok ->    ok;
- 	Error -> erlang:fault(Error, [F,Format,Args])
+ 	Error -> erlang:error(Error, [F,Format,Args])
     end.
 
 println(F, Format, Args) ->
     case file:write(F, [io_lib:format(Format, Args),io_lib:nl()]) of
 	ok ->    ok;
-	Error -> erlang:fault(Error, [F,Format,Args])
+	Error -> erlang:error(Error, [F,Format,Args])
     end.
 
 close(F) ->
     case file:close(F) of
 	ok ->    ok;
-	Error -> erlang:fault(Error, [F])
+	Error -> erlang:error(Error, [F])
     end.
 
 %% Convert certain terms to printable strings in a
@@ -1251,7 +1255,7 @@ format(false) ->    "false";
 format(A) when is_atom(A) ->    atom_to_list(A);
 format(L) when is_list(L) ->    L.
 
-format_decimals(F) when float(F), F >= 0.0 ->
+format_decimals(F) when is_float(F), F >= 0.0 ->
     format_decimals_1(F).
 
 format_decimals_1(0.0) ->    ".0";
