@@ -8,7 +8,7 @@
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: wpc_wrl.erl,v 1.19 2006/01/08 18:02:00 giniu Exp $
+%%     $Id: wpc_wrl.erl 211 2008-01-06 08:06:06Z bjorng $
 %%
 
 -module(wpc_wrl).
@@ -278,7 +278,7 @@ lookup(K, L) ->
 % If the first char is not allowed
 % then prefix whole id with W. For rest of not allowed chars
 % turn them into a safe 2 char representation.
-clean_id(Id) when atom(Id) ->
+clean_id(Id) when is_atom(Id) ->
     clean_id(atom_to_list(Id));
 clean_id([First|T]) ->
     case is_not_allowed_first_char(First) of
