@@ -8,7 +8,7 @@
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: wings_edge_cmd.erl,v 1.9 2006/10/22 16:33:54 giniu Exp $
+%%     $Id: wings_edge_cmd.erl 264 2008-02-28 15:50:13Z optigon $
 %%
 
 -module(wings_edge_cmd).
@@ -61,11 +61,11 @@ menu(X, Y, St) ->
     wings_menu:popup_menu(X, Y, edge, Menu).
 
 dslv() ->
-	fun
-		(1, _Ns) -> {edge,dissolve};
-		(3, _Ns) -> {edge,clean_dissolve};
-		(_, _) -> ignore
-	end.
+    fun
+	(1, _Ns) -> {edge,dissolve};
+	(3, _Ns) -> {edge,clean_dissolve};
+	(_, _) -> ignore
+    end.
 
 cut_line(#st{sel=[{_,Es}]}) ->
     case gb_sets:size(Es) of
@@ -119,8 +119,8 @@ command(cut_pick, St) ->
     cut_pick(St);
 command({cut,ask}, St) ->
     wings_ask:ask(cut_command(),
-                  [{?__(1,"Segments"), 2}],
-                  fun([Ret]) -> cut(Ret, St) end);
+		  [{?__(1,"Segments"), 2}],
+		  fun([Ret]) -> cut(Ret, St) end);
 command({cut,Num}, St) ->
     {save_state,cut(Num, St)};
 command(connect, St) ->
@@ -147,7 +147,6 @@ command(vertex_color, St) ->
     wings_color:choose(fun(Color) ->
 			       set_color(Color, St)
 		       end).
-
 %%%
 %%% The Connect command.
 %%%
@@ -309,12 +308,14 @@ cut_pick_marker({finish,[I]}, D0, Edge, We, Start, Dir, Char) ->
 %%%
 
 clean_dissolve(St0) ->
-	St = wings_sel:map(fun(Es, We) ->
-	We1 = wings_edge:dissolve_edges(Es, We),
-    IsolatedVs1 = wings_vertex:isolated(We1),
-    wings_edge:dissolve_isolated_vs(IsolatedVs1, We1)
-	end, St0),
-	wings_sel:clear(St).
+    St = wings_sel:map(fun(Es, We) ->
+ 				     IsolatedVs1 = wings_vertex:isolated(We),
+ 				     We1 = wings_edge:dissolve_edges(Es, We),
+ 				     IsolatedVs2 = wings_vertex:isolated(We1),
+ 				     C = IsolatedVs2 -- IsolatedVs1,
+ 				     wings_edge:dissolve_isolated_vs(C, We1)
+		       end, St0),
+    wings_sel:clear(St).
 
 %%%
 %%% The Dissolve command.
@@ -350,12 +351,12 @@ slide(St) ->
     Stop = wings_pref:get_value(slide_stop, false),
     State = {Mode,none,Stop},
     SUp = SDown = SN = SBi = {0.0,0.0,0.0},
-    {Tvs,_,_,_,_,MinUp,MinDw} = 
+    {Tvs,_,_,_,_,MinUp,MinDw} =
 	wings_sel:fold(
 	  fun(EsSet, #we{id=Id} = We, {Acc,Up0,Dw0,N0,Bi0,MinUp,MinDw}) ->
 		  LofEs0 = wings_edge_loop:partition_edges(EsSet, We),
 		  LofEs = reverse(sort([{length(Es),Es} || Es <- LofEs0])),
-		  {{Slides,MUp,MDw},Up,Dw,N,Bi} = 
+		  {{Slides,MUp,MDw},Up,Dw,N,Bi} =
 		      slide_setup_edges(LofEs,Up0,Dw0,N0,Bi0,We,
 					{gb_trees:empty(),MinUp,MinDw}),
 		  {[{Id,make_slide_tv(Slides, State)}|Acc],Up,Dw,N,Bi,MUp,MDw}
@@ -365,25 +366,25 @@ slide(St) ->
     wings_drag:setup(Tvs, Units, Flags, St).
 
 slide_mode(MinUp,MinDw) ->
-    fun(help, State)              ->    slide_help(State);
-       ({key,$1}, {relative,F,S}) ->    {absolute,F,S};
-       ({key,$1}, {absolute,F,S}) ->    {relative,F,S};
+    fun(help, State)		  ->	slide_help(State);
+       ({key,$1}, {relative,F,S}) ->	{absolute,F,S};
+       ({key,$1}, {absolute,F,S}) ->	{relative,F,S};
        ({key,$2}, {Mode,none,S})  ->
 	    case get(wings_slide) of
-		undefined -> 
+		undefined ->
 		    {Mode,none,S};
-		Dx when Dx >= 0 -> 
+		Dx when Dx >= 0 ->
 		    {Mode,positive,S};
-		_ -> 
+		_ ->
 		    {Mode,negative,S}
 	    end;
-       ({key,$2}, {Mode,_,S})     ->    {Mode,none,S};
-       ({key,$3}, {Mode,F,false}) ->    {Mode,F,true};
-       ({key,$3}, {Mode,F,true})  ->    {Mode,F,false};
+       ({key,$2}, {Mode,_,S})	  ->	{Mode,none,S};
+       ({key,$3}, {Mode,F,false}) ->	{Mode,F,true};
+       ({key,$3}, {Mode,F,true})  ->	{Mode,F,false};
 
        (units, NewState) ->
 	    slide_units(NewState,MinUp,MinDw);
-       
+
        (done, {NewMode,_,NewStop})->
 	    wings_pref:set_value(slide_mode, NewMode),
 	    wings_pref:set_value(slide_stop, NewStop),
@@ -407,10 +408,10 @@ slide_help_mode(relative) -> ?__(1,"Absolute");
 slide_help_mode(absolute) -> ?__(2,"Relative").
 
 slide_help_freeze(none)   -> ?STR(slide_help_mode,3,"Freeze direction");
-slide_help_freeze(_)      -> ?STR(slide_help_mode,4,"Thaw direction").
+slide_help_freeze(_)	  -> ?STR(slide_help_mode,4,"Thaw direction").
 
-slide_help_stop(false)    -> ?STR(slide_help_mode,5,"Stop at other edges");
-slide_help_stop(true)     -> ?STR(slide_help_mode,6,"Continue past other edges").
+slide_help_stop(false)	  -> ?STR(slide_help_mode,5,"Stop at other edges");
+slide_help_stop(true)	  -> ?STR(slide_help_mode,6,"Continue past other edges").
 
 make_slide_tv(Slides, State) ->
     Vs = gb_trees:keys(Slides),
@@ -418,14 +419,14 @@ make_slide_tv(Slides, State) ->
 
 %% The calculating fun
 slide_fun(Dx0,{Mode,Freeze,_Stop}, Slides) ->
-    {Dx,I} = 
-	case Freeze of   %% 3 = UP, 2 = Down
+    {Dx,I} =
+	case Freeze of	 %% 3 = UP, 2 = Down
 	    none when Dx0 >= 0 -> {Dx0, 3};
-	    none ->               {-Dx0,2};
+	    none ->		  {-Dx0,2};
 	    positive -> 	  {Dx0, 3};
-	    negative ->           {-Dx0,2}
+	    negative -> 	  {-Dx0,2}
 	end,
-    case Mode of 
+    case Mode of
 	relative ->
 	    fun(V,A) ->
 		    Slide = gb_trees:get(V, Slides),
@@ -461,12 +462,12 @@ slide_setup_edges([], Up,Dw,N,Bi,_,Slides) ->
     {Slides,Up,Dw,N,Bi}.
 
 slide_add_edges([{LUp,LDw,LN,Es}|Parts],GUp,GDw,GN,GBi,Acc0) ->
-    %%    io:format("UDN ~p ~p ~p ~p~n", [len(LUp), len(LDw), len(LN), length(Es)]),
-    Count  = length(Es)/2,    
+    %%	  io:format("UDN ~p ~p ~p ~p~n", [len(LUp), len(LDw), len(LN), length(Es)]),
+    Count  = length(Es)/2,
     Vec1   = norm(LUp), LenUp = len(LUp),
     Vec2   = norm(LDw), LenDw = len(LDw),
-    Vec3   = norm(LN),  LenN  = len(LN),
-    Bi     = norm(cross(Vec1,Vec3)),
+    Vec3   = norm(LN),	LenN  = len(LN),
+    Bi	   = norm(cross(Vec1,Vec3)),
     Rotation = dot(Bi, norm(GBi)),
 %    io:format("BIs ~p ~p ~p ~p ~n", [Bi, norm(GBi), Rotation, Count]),
     case (LenUp/Count > 0.707) and (LenN/Count > 0.707) and
@@ -478,9 +479,9 @@ slide_add_edges([{LUp,LDw,LN,Es}|Parts],GUp,GDw,GN,GBi,Acc0) ->
 	    case (LenUp >= 1) or (LenDw >= 1)
 		or (LenN =< 1) or (Count < 4) of
 		true -> %% Make sure up is up..
-		    DotUp1 = dot(Vec1,norm(GUp)), 
+		    DotUp1 = dot(Vec1,norm(GUp)),
 		    %%DotDw1 = dot(Vec1,norm(GDw)),
-		    %%DotUp2 = dot(Vec2,norm(GUp)), 
+		    %%DotUp2 = dot(Vec2,norm(GUp)),
 		    DotDw2 = dot(Vec2,norm(GDw)),
 		    if (DotUp1 >= 0) and (DotDw2 >= 0) ->
 			    Acc  = slide_dirs(Es,1.0,Acc0),
@@ -496,7 +497,7 @@ slide_add_edges([{LUp,LDw,LN,Es}|Parts],GUp,GDw,GN,GBi,Acc0) ->
 			    slide_add_edges(Parts,add(Vec2,GUp),
 					    add(Vec1,GDw),GN,add(GBi,Bi),Acc)
 		    end;
-		false -> 
+		false ->
 		    %% Probably a loop, make sure it goes in/out and not out/in.
 		    %% BUGBUG this isn't good enough..
 		    Norm0 = Vec3,
@@ -535,7 +536,7 @@ slide_part_loop(Es,We) ->
     [slide_dir(P,Eis,Def,Def,Def,[]) || P <- Parts].
 
 slide_gather_info([Edge|Es],We=#we{es=Etab,vp=Vtab},Acc) ->
-    #edge{vs=V1,ve=V2,ltpr=LP,ltsu=LS,lf=LF,rtpr=RP,rtsu=RS,rf=RF} = 
+    #edge{vs=V1,ve=V2,ltpr=LP,ltsu=LS,lf=LF,rtpr=RP,rtsu=RS,rf=RF} =
 	gb_trees:get(Edge, Etab),
     A1 = other(V1,gb_trees:get(RP, Etab)),
     B1 = other(V1,gb_trees:get(LS, Etab)),
@@ -547,7 +548,7 @@ slide_gather_info([Edge|Es],We=#we{es=Etab,vp=Vtab},Acc) ->
     A1pos = gb_trees:get(A1, Vtab),A2pos = gb_trees:get(A2, Vtab),
     B1pos = gb_trees:get(B1, Vtab),B2pos = gb_trees:get(B2, Vtab),
     E1v1  = sub(A1pos,V1pos), E2v1 = sub(B1pos,V1pos),
-    E1v2  = sub(A2pos,V2pos), E2v2 = sub(B2pos,V2pos), 
+    E1v2  = sub(A2pos,V2pos), E2v2 = sub(B2pos,V2pos),
     NE1v1 = norm(E1v1), NE2v1 = norm(E2v1),
     NE1v2 = norm(E1v2), NE2v2 = norm(E2v2),
 
@@ -578,7 +579,7 @@ find_edge({Edge,V1,V2},Es) ->
 other(Vertex, #edge{vs=Vertex,ve=Other}) -> Other;
 other(Vertex, #edge{vs=Other,ve=Vertex}) -> Other.
 
-swap({Vpos,Ndir,Pdir}) -> 
+swap({Vpos,Ndir,Pdir}) ->
     {Vpos,Pdir,Ndir}.
 
 add_slide_vertex(V,{Vpos,{Ndir,NL},{Pdir,PL}},Acc) ->
