@@ -3,12 +3,12 @@
 %%
 %%     This module implements the Help menu.
 %%
-%%  Copyright (c) 2001-2005 Bjorn Gustavsson
+%%  Copyright (c) 2001-2008 Bjorn Gustavsson
 %%
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: wings_help.erl,v 1.83 2005/06/08 05:36:50 bjorng Exp $
+%%     $Id: wings_help.erl 290 2008-03-09 18:02:40Z optigon $
 %%
 
 -module(wings_help).
@@ -148,7 +148,7 @@ advanced_menus(Head) ->
     Help = [?__(1,"In the ")++
 	    edit_prefs()++
 	    ?__(4,
-		 "dialog, there is a check box for \"Advanced Menus\"."),
+		 " dialog, there is a check box for \"Advanced Menus\"."),
 	    ?__(5,
 		 "Activating advanced menus provide the following additional features:"),
 	    ?__(6,"New commands: ")
@@ -158,7 +158,7 @@ advanced_menus(Head) ->
 	    cmd([?__(10,"Face"),
 		 ?__(11,"Lift")
 		]),
-	    ?__(12,"Vector based operations: The means to specify an axis (or vector) and to be able to re-locate it so it passes though a new point."),
+	    ?__(12,"Vector based operations: The means to specify an axis (or vector) and to be able to re-locate it so it passes through a new point."),
 	    ?__(13,"Magnet operations: A facility that provides a way of attaining smooth modifications / transitions to surrounding geometry during the operation of any valid tool. Magnet type and influence radius settings provide further control options."),
 
 	    ?__(14,"Vector and magnet operations can be combined."),
@@ -248,8 +248,22 @@ hotkeys(Head) ->
 
 def_hotkeys(Head) ->
     Help = [?__(1,"Any command that appears in a menu, can be assigned a keyboard short-cut (hotkey)."),
-    	    ?__(2,"To assign a hotkey to a command, open the menu containing the command. With the command high-lighted, press the [Insert] or [/] key, and then press the key you want to assign the command to."),
-	    ?__(3,"To delete a hotkey, similarly high-light the command in a menu, and press the [Del] or [\\] key.")],
+	    ?__(4,"To assign a hotkey to a command:"),
+	    ?__(5,"1. Open the menu containing the command and high-light the command."),
+	    ?__(6,"2. Press the [Insert] or [/] key."),
+	    ?__(7,"3. If the command does different things depending "
+		"on which mouse button is pressed (when Advanced menus are enabled), "
+		"there will be a message in the information line asking you to "
+		"press a mouse button to choose which variant of the command to "
+		"assign to the hotkey. (Note: The mouse cursor must remain inside "
+		"the menu when you click.)"),
+	    ?__(8,"4. The information line ask you to press the key that the command "
+		"should be assigned to."),
+
+ 	    ?__(9,"To delete a hotkey, similarly high-light the command in a menu,"
+		" and press the [Del] or [\\] key. A dialog box listing all keys "
+		"bound to the command will appear. "
+		"Check all hotkeys you want to delete.")],
     help_window(Head, Help).
 
 lights(Head) ->
@@ -265,11 +279,16 @@ lights(Head) ->
 opengl_info(Head) ->
     gl:getError(),			%Clear any previous error.
     [{_,VerTuple}] = ets:lookup(wings_gl_ext, version),
+    {SdlVersion0,_} = sdl_video:wm_getInfo(),
+    SdlVersion1 = [integer_to_list(V) || V <- tuple_to_list(SdlVersion0)],
+    SdlVersion = string:join(SdlVersion1, "."),
     Help = [
 	    ?__(1,"Vendor: ") ++ gl:getString(?GL_VENDOR) ++ "\n" ++
 	    ?__(2,"Renderer: ") ++ gl:getString(?GL_RENDERER) ++ "\n" ++
-	    ?__(3,"Version: ") ++ gl:getString(?GL_VERSION),
-	    ?__(4,"Version tuple: ") ++ lists:flatten(io_lib:format("~p\n", [VerTuple])),
+	    ?__(3,"Version: ") ++ gl:getString(?GL_VERSION) ++ "\n" ++
+	    ?__(4,"Version tuple: ") ++ lists:flatten(io_lib:format("~p\n", [VerTuple])) ++
+	    "SDL: " ++ SdlVersion,
+
 	    get_info([{?__(5,"Red bits"),?GL_RED_BITS},
 		      {?__(6,"Green bits"),?GL_GREEN_BITS},
 		      {?__(7,"Blue bits"),?GL_BLUE_BITS},
