@@ -3,12 +3,12 @@
 %%
 %%     This module manages images.
 %%
-%%  Copyright (c) 2003-2004 Bjorn Gustavsson
+%%  Copyright (c) 2003-2008 Bjorn Gustavsson
 %%
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: wings_image.erl,v 1.53 2006/01/25 21:15:12 dgud Exp $
+%%     $Id: wings_image.erl 185 2008-01-01 14:39:08Z bjorng $
 %%
 
 -module(wings_image).
@@ -43,7 +43,47 @@ image_formats() ->
     wings_plugin:call_ui({image,formats,[]}).
 
 image_read(Ps) ->
-    wings_plugin:call_ui({image,read,Ps}).
+    CurrDir  = wings_pref:get_value(current_directory),
+    OptDir   = proplists:get_value(opt_dir,  Ps, undefined),
+    AbsFile  = proplists:get_value(filename, Ps),
+    FileName = filename:basename(AbsFile),
+    Dirs     = filename:split(filename:dirname(AbsFile)),    
+    case wings_plugin:call_ui({image,read,Ps}) of
+	{error, enoent} -> 	    
+	    try_relative_paths(Dirs,FileName,OptDir,CurrDir,Ps);
+	Ok = #e3d_image{} ->
+	    Ok#e3d_image{filename=AbsFile};
+	Other ->
+	    Other
+    end.
+		
+try_relative_paths(Dirs,FileName,undefined,CurrDir,Ps) ->
+    try_relative_paths(CurrDir,Dirs,FileName,Ps);
+try_relative_paths(Dirs,FileName,OptDir,CurrDir,Ps) ->
+    case try_relative_paths(OptDir,Dirs,FileName,Ps) of
+	{error, enoent} ->
+	    try_relative_paths(Dirs,FileName,undefined,CurrDir,Ps);
+	Other -> 
+	    Other
+    end.
+
+try_relative_paths(Start, Rel, File, Ps0) ->
+    Abs = case Rel of 
+	      [] -> filename:join(Start,File);
+	      _  -> filename:join(filename:join(Start,filename:join(Rel)),File)
+	  end,
+    Ps = lists:keyreplace(filename,1,Ps0,{filename,Abs}),
+    case wings_plugin:call_ui({image,read,Ps}) of
+	Err = {error, enoent} ->
+	    case Rel of
+		[] -> Err;
+		[_|Rest] -> try_relative_paths(Start,Rest,File,Ps0)
+	    end;
+	Ok = #e3d_image{} ->
+	    Ok#e3d_image{filename=Abs};
+	Other ->
+	    Other
+    end.
 
 image_write(Ps) ->
     case catch wings_plugin:call_ui({image,write,Ps}) of
@@ -62,7 +102,7 @@ from_file(Filename) ->
     case image_read(Props) of
 	#e3d_image{}=Image ->
 	    Name = filename:rootname(filename:basename(Filename)),
-	    req({new,Image#e3d_image{filename=Filename,name=Name},false});
+	    req({new,Image#e3d_image{name=Name},false});
 	{error,_}=Error -> Error
     end.
 
@@ -299,7 +339,8 @@ create_bump(Id, BumpId, #ist{images=Images0}) ->
 						  ?GL_UNSIGNED_BYTE,Bits),
 				    New
 			    end,
-		    MipMaps = e3d_image:buildNormalMipmaps(Image),
+		    MipMapFun = fun() -> e3d_image:buildNormalMipmaps(Image) end,
+		    MipMaps = worker_process(MipMapFun),
 		    TxId = BumpId;
 		_ ->
 		    %% Scale ?? 4 is used in the only example I've seen.
@@ -350,7 +391,7 @@ create_pnoise() ->
 	gl:bindTexture(?GL_TEXTURE_3D, NoiseMap),
 	Map = pnoise:s_map3d(128),
 
-	Assert = size(Map),
+	Assert = byte_size(Map),
 	Assert = 128*128*128*4,
 
 	gl:texParameteri(?GL_TEXTURE_3D, ?GL_TEXTURE_MAG_FILTER, ?GL_LINEAR),
@@ -478,24 +519,11 @@ texture_format(#e3d_image{type=b8g8r8a8}) -> ?GL_BGRA;
 texture_format(#e3d_image{type=g8}) -> ?GL_LUMINANCE;
 texture_format(#e3d_image{type=a8}) -> ?GL_ALPHA.
 
-internal_format(Type) ->
-    %% Compression lowers the quality, especially when bump/normal maps
-    %% are compressed. Disabled for now.
-%     Compress = wings_util:is_gl_ext({1,3}, 'GL_ARB_texture_compression'),
-    internal_format(Type, false).
-
-internal_format(?GL_BGR, false) -> ?GL_RGB;
-internal_format(?GL_BGRA, false) -> ?GL_RGBA;
-internal_format(Else, false) -> Else;
-internal_format(?GL_BGR, true) -> ?GL_COMPRESSED_RGB;
-internal_format(?GL_BGRA, true) -> ?GL_COMPRESSED_RGBA;
-internal_format(?GL_ALPHA, true) -> ?GL_COMPRESSED_ALPHA;
-internal_format(?GL_LUMINANCE, true) -> ?GL_COMPRESSED_LUMINANCE;
-internal_format(?GL_LUMINANCE_ALPHA, true) -> ?GL_COMPRESSED_LUMINANCE_ALPHA;
-internal_format(?GL_INTENSITY, true) -> ?GL_COMPRESSED_INTENSITY;
-internal_format(?GL_RGB, true) ->  ?GL_COMPRESSED_RGB;
-internal_format(?GL_RGBA, true) -> ?GL_COMPRESSED_RGBA;  
-internal_format(Else, _) -> Else.
+%% Long ago we tried to use compression, but we no longer do since compression
+%% lowers the quality too much, especially for bump/normal maps.
+internal_format(?GL_BGR) -> ?GL_RGB;
+internal_format(?GL_BGRA) -> ?GL_RGBA;
+internal_format(Else) -> Else.
 
 delete(Id, #ist{images=Images0}=S) ->
     delete_bump(Id),
@@ -774,7 +802,7 @@ create_image() ->
 
 create_image_1(Pattern, W, H) ->
     Pixels = pattern(Pattern, W, H),
-    case {size(Pixels),3*W*H} of
+    case {byte_size(Pixels),3*W*H} of
 	{S,S} ->				%Assertion.
 	    Im = #e3d_image{width=W,height=H,image=Pixels,order=upper_left},
 	    new(atom_to_list(Pattern), Im)
@@ -904,3 +932,19 @@ get_cube_vec(Side, Size, X, Y) ->
 	end,
     {RX,RY,RZ} = e3d_vec:norm(Vec),
     [round(128+127*RX),round(128+127*RY),round(128+127*RZ)].
+
+%% Run a computation in a worker process with a generous heap size
+%% and the default generational garbage collector. Before R12B,
+%% heap fragments would allow the heap to be over-committed, but in
+%% R12B there will be garbage collection as soon as the heap space
+%% is exhausted.
+worker_process(WorkFun) ->
+    ResultRef = make_ref(),
+    {Pid,Ref} = spawn_opt(fun() -> exit({ResultRef,WorkFun()}) end,
+			  [monitor,{min_heap_size,20000}]),
+    receive
+	{'DOWN',Ref,process,Pid,{ResultRef,Res}} ->
+	    Res;
+	{'DOWN',Ref,process,Pid,Reason} ->
+	    exit(Reason)
+    end.
