@@ -4,12 +4,12 @@
 %%     Implementation of languages.
 %%
 %%  Copyright (c) 2004 Riccardo Venier, Dan Gudmundsson
-%%                2004-2005 Dan Gudmundsson, Bjorn Gustavsson
+%%                2004-2008 Dan Gudmundsson, Bjorn Gustavsson
 %%
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: wings_lang.erl,v 1.21 2006/08/07 01:41:17 antoneos Exp $
+%%     $Id: wings_lang.erl 180 2008-01-01 13:58:37Z bjorng $
 %%
 %%  Totally rewritten but Riccardo is still the one who did the hard work.
 %%
@@ -64,7 +64,7 @@ load_language_only(Lang0) when is_list(Lang0) ->
 	?DEF_LANG_ATOM -> ok;
 	_  ->
 	    ets:new(?MODULE, [named_table]),
-	    Root = code:lib_dir(wings),
+	    Root = wings_util:lib_dir(wings),
 	    LangFile = "_"++Lang0++".lang",
 	    load_language(Root, ?LANG_DIRS, LangFile)
     end.
@@ -115,7 +115,7 @@ load_file([latin1|Trans]) ->
     load_file_1(latin1, Trans);
 load_file(['iso-8859-1'|Trans]) ->
     load_file_1(latin1, Trans);
-load_file([Atom|Trans]) when is_atom(Trans) ->
+load_file([Atom|Trans]) when is_atom(Atom) ->
     io:format("Ignoring: ~p\n", [Atom]),
     load_file_1(default, Trans);
 load_file(Trans) -> load_file_1(default, Trans).
@@ -170,7 +170,7 @@ load_file_2(Trans) ->
     Trav(Trans,[],Trav).
 
 available_languages() ->
-    Root = code:lib_dir(wings),
+    Root = wings_util:lib_dir(wings),
     Files = 
 	filelib:wildcard(filename:join([Root,"src","*.lang"])) ++ 
 	filelib:wildcard(filename:join([Root,"ebin","*.lang"])),
