@@ -8,7 +8,7 @@
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: wings_pick.erl,v 1.149 2005/04/12 20:25:19 bjorng Exp $
+%%     $Id: wings_pick.erl 292 2008-03-14 00:22:28Z optigon $
 %%
 
 -module(wings_pick).
@@ -94,19 +94,35 @@ pick(X, Y, _, St0) ->
 %%
 %% Highlighting on mouse move.
 %%
-
 get_hilite_event(HL) ->
     fun(Ev) -> handle_hilite_event(Ev, HL) end.
 
-handle_hilite_event(redraw, #hl{redraw=#st{sel=[]}=St,prev={_,Where,{_,Elem}}}) ->
+handle_hilite_event(redraw,#hl{redraw=#st{sel=[]}=St,prev={SelMode,Where,{Obj,Elem}}}=Hl) ->
+    Mode = case SelMode of
+      vertex -> ?__(4,"Vertex");
+      edge -> ?__(5,"Edge");
+      face -> ?__(6,"Face");
+      body -> none
+    end,
     Info = case Where of
-	       original ->
-		   io_lib:format("#~p", [Elem]);
-	       mirror ->
-		   wings_util:format("#~p ~s", 
-				     [Elem,
-				      ?__(2,"(in mirror)")])
-	   end,
+      original ->
+        case SelMode of
+          body ->
+            wings_util:format("~s #~p", [?__(3,"Object"),Obj]);
+          _Other ->
+            enhanced_hl_info(wings_util:format("~s #~p, ~s #~p",
+               [?__(3,"Object"),Obj,Mode,Elem]),Hl)
+        end;
+      mirror ->
+        case SelMode of
+          body ->
+            wings_util:format("~s #~p ~s", [?__(3,"Object"),
+              Obj,?__(2,"(in mirror)")]);
+          _Other ->
+            enhanced_hl_info(wings_util:format("~s #~p, ~s #~p ~s",
+            [?__(3,"Object"),Obj,Mode,Elem,?__(2,"(in mirror)")]),Hl)
+        end
+      end,
     wings:redraw(Info, St),
     keep;
 handle_hilite_event(redraw, #hl{redraw=#st{}=St}) ->
@@ -221,6 +237,54 @@ hilit_draw_sel(body, _, #dlo{src_we=We}=D) ->
     gl:'end'(),
     gl:disable(?GL_POLYGON_STIPPLE).
 
+enhanced_hl_info(Base,#hl{redraw=#st{sel=[],shapes=Shs},prev={SelMode,_,{Obj,Elem}}})->
+    case wings_pref:get_value(info_text_on_hilite) of
+      true ->
+        We = gb_trees:get(Obj, Shs),
+        case SelMode of
+          vertex ->
+            {X,Y,Z} = wings_vertex:pos(Elem, We),
+            [Base|io_lib:format(?__(1,". Position <~s  ~s  ~s>"),
+                               [wings_util:nice_float(X),
+                                wings_util:nice_float(Y),
+                                wings_util:nice_float(Z)])];
+          edge -> 
+            #edge{vs=Va,ve=Vb} = gb_trees:get(Elem, We#we.es),
+            {Xa,Ya,Za} = wings_vertex:pos(Va, We),
+            {Xb,Yb,Zb} = wings_vertex:pos(Vb, We),
+            Length = e3d_vec:dist({Xa,Ya,Za}, {Xb,Yb,Zb}),
+            {X,Y,Z} = e3d_vec:average({Xa,Ya,Za}, {Xb,Yb,Zb}),
+            [Base|io_lib:format(?__(3,". Midpoint <~s  ~s  ~s>\nLength ~s") ++
+                                "  <~s  ~s  ~s>",
+                                [wings_util:nice_float(X),
+                                 wings_util:nice_float(Y),
+                                 wings_util:nice_float(Z),
+                                 wings_util:nice_float(Length),
+                                 wings_util:nice_float(abs(Xb - Xa)),
+                                 wings_util:nice_float(abs(Yb - Ya)),
+                                 wings_util:nice_float(abs(Zb - Za))])];
+          face ->
+            {X,Y,Z} = wings_face:center(Elem, We),
+            Area = area_info(Elem, We),
+            Mat = wings_facemat:face(Elem, We),
+            [Base|io_lib:format(?__(4,". Midpoint <~s  ~s  ~s> \nMaterial ~s.")
+                                ++ Area,
+                                [wings_util:nice_float(X),
+                                 wings_util:nice_float(Y),
+                                 wings_util:nice_float(Z),
+                                 Mat])]
+         end;
+      false ->
+        Base
+    end.
+
+area_info(Face, We) ->
+    case wings_face:vertices(Face,We) =< 50 of
+      true -> A = wings_face:area(Face, We),
+              wings_util:format(?__(40," Area ~s"), [wings_util:nice_float(A)]);
+      false -> []
+    end.
+
 %%
 %% Marquee picking.
 %%
