@@ -3,12 +3,12 @@
 %%
 %%     Preference management.
 %%
-%%  Copyright (c) 2001-2004 Bjorn Gustavsson
+%%  Copyright (c) 2001-2008 Bjorn Gustavsson
 %%
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: wings_pref.erl,v 1.138 2006/07/28 18:54:27 antoneos Exp $
+%%     $Id: wings_pref.erl 283 2008-03-09 01:24:24Z optigon $
 %%
 
 -module(wings_pref).
@@ -20,7 +20,7 @@
 	 set_scene_value/2,set_scene_default/2,
 	 delete_scene_value/0,delete_scene_value/1]).
 
--define(NEED_ESDL, 1).    %% Some keybindings
+-define(NEED_ESDL, 1).	  %% Some keybindings
 -include("wings.hrl").
 -import(lists, [foreach/2,keysearch/3,map/2,reverse/1,sort/1,last/1,
 		foldl/3]).
@@ -39,7 +39,7 @@ load() ->
     case old_pref_file() of
 	none -> ok;
 	PrefFile ->
-            io:format("Reading preferences from: ~s\n", [PrefFile]),
+	    io:format("Reading preferences from: ~s\n", [PrefFile]),
 	    case file:consult(PrefFile) of
 		{ok,List0} ->
 		    List = clean(List0),
@@ -50,6 +50,7 @@ load() ->
     end.
 
 finish() ->
+    win32_save_maximized(),
     foreach(fun({Key,Val}) ->
 		    set_value(Key, Val)
 	    end, ets:tab2list(wings_delayed_update)),
@@ -58,9 +59,9 @@ finish() ->
     List = prune_defaults(List0),
     Format = "~p. \n",
     PostProcess = case os:type() of
-                      {win32,_} -> fun insert_crs/1;
-                      _ -> fun(L) -> L end
-                  end,
+		      {win32,_} -> fun insert_crs/1;
+		      _ -> fun(L) -> L end
+		  end,
     Write = fun({{bindkey,_},_,default}) -> [];
 	       ({{bindkey,_},_,plugin}) -> [];
 	       ({{bindkey,_,_},_,default}) -> [];
@@ -96,6 +97,14 @@ insert_crs(C) when is_integer(C) -> C.
 prune_defaults(List) ->
     List -- defaults().
 
+win32_save_maximized() ->
+    case os:type() of
+	{win32,_} ->
+	    set_value(win32_start_maximized, sdl_video:wm_isMaximized());
+	_ ->
+	    ok
+    end.
+
 %%%
 %%% Search for a pre-existing preference file.
 %%%
@@ -124,44 +133,44 @@ win32_pref() ->
 %% and "My Documents".
 win32_pref_1(R, [FolderType|T]) ->
     case win32_special_folder(R, FolderType) of
-        none -> win32_pref_1(R, T);
-        Path ->
-            case try_location(Path, ?WIN32_PREFS) of
-                none -> win32_pref_1(R, T);
-                File -> File
-            end
+	none -> win32_pref_1(R, T);
+	Path ->
+	    case try_location(Path, ?WIN32_PREFS) of
+		none -> win32_pref_1(R, T);
+		File -> File
+	    end
     end;
 win32_pref_1(R, []) ->
-    case try_location(code:lib_dir(wings), ?WIN32_PREFS) of
-        none -> win32_pref_2(R);
-        File -> File
+    case try_location(wings_util:lib_dir(wings), ?WIN32_PREFS) of
+	none -> win32_pref_2(R);
+	File -> File
     end.
-            
+
 %% No preferences found so far. Search in old installations of
 %% Wings for preference files.
 win32_pref_2(R) ->
     case win32_9816(R) of
-        none -> win32_pref_pre9816(R);
-        File -> File
+	none -> win32_pref_pre9816(R);
+	File -> File
     end.
 
 win32_9816(R) ->
     %% Search for a preference file in a Wings installation in 0.98.16.
     %% (Too bad... in a special place in this release only.)
     case win32reg:change_key(R, "\\hklm\\SOFTWARE\\Wings 3D") of
-        ok ->
-            case win32reg:sub_keys(R) of
-                {ok,SubKeys0} ->
-                    SubKeys = reverse(sort(SubKeys0)),
-                    {ok,Curr} = win32reg:current_key(R),
-                    win32_9816_1(R, SubKeys, Curr);
-                {error,_Error} ->
+	ok ->
+	    case win32reg:sub_keys(R) of
+		{ok,SubKeys0} ->
+		    SubKeys = reverse(sort(SubKeys0)),
+		    {ok,Curr} = win32reg:current_key(R),
+		    win32_9816_1(R, SubKeys, Curr);
+		{error,_Error} ->
 		    %% Can't read sub keys - strange.
-                    none
-            end;
-        {error,_Error} ->
+		    none
+	    end;
+	{error,_Error} ->
 	    %% No 'Wings 3D' key (this is STRANGE)
-            none
+	    none
     end.
 
 win32_9816_1(R, [K|Keys], Curr) ->
@@ -170,8 +179,8 @@ win32_9816_1(R, [K|Keys], Curr) ->
     ok = win32reg:change_key(R, Curr),
     WingsDirs = filelib:wildcard(Dir++"/lib/wings-*"),
     case try_locations(WingsDirs, ?WIN32_OLD_PREFS) of
-        none -> win32_9816_1(R, Keys, Curr);
-        File -> File
+	none -> win32_9816_1(R, Keys, Curr);
+	File -> File
     end;
 win32_9816_1(_, [], _) -> none.
 
@@ -179,43 +188,43 @@ win32_pref_pre9816(R) ->
     %% Search for a preference file in a Wings installation older than 0.98.16
     %% using the uninstall string.
     case win32reg:change_key(R, "\\hklm\\SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Uninstall\\Wings 3D") of
-        ok ->
-            case win32reg:value(R, "UninstallString") of
-                {ok,Str0} ->
-                    Str = strip_quotes(Str0),
-                    try_location(filename:dirname(Str), ?WIN32_OLD_PREFS);
-                {error,_} -> none
-            end;
-        {error,_} -> none
+	ok ->
+	    case win32reg:value(R, "UninstallString") of
+		{ok,Str0} ->
+		    Str = strip_quotes(Str0),
+		    try_location(filename:dirname(Str), ?WIN32_OLD_PREFS);
+		{error,_} -> none
+	    end;
+	{error,_} -> none
     end.
 
 win32_special_folder(R, FolderType) ->
     Key = "\\hkcu\\Software\\Microsoft\\Windows\\CurrentVersion\\Explorer\\Shell Folders",
     case win32reg:change_key(R, Key) of
-        ok ->
-            case win32reg:value(R, FolderType) of
-                {error,_} -> error;
-                {ok,Value} -> Value
-            end;
-        _ -> error
+	ok ->
+	    case win32reg:value(R, FolderType) of
+		{error,_} -> none;
+		{ok,Value} -> Value
+	    end;
+	_ -> error
     end.
 
 reg_get_default(R) ->
     %% There seems to be a bug in win32reg:value/2 preventing
     %% us from retrieving the default value. Workaround follows.
     case win32reg:values(R) of
-        {ok,Values} ->
-            case keysearch(default, 1, Values) of
-                {value,{default,Val}} -> Val;
-                false -> ""
-            end;
-        {error,_} -> ""
+	{ok,Values} ->
+	    case keysearch(default, 1, Values) of
+		{value,{default,Val}} -> Val;
+		false -> ""
+	    end;
+	{error,_} -> ""
     end.
-    
+
 strip_quotes([$"|T0]) ->
     case reverse(T0) of
-        [$"|T] -> reverse(T);
-        _ -> T0
+	[$"|T] -> reverse(T);
+	_ -> T0
     end;
 strip_quotes(S) -> S.
 
@@ -230,7 +239,7 @@ new_pref_file() ->
 	{unix,_} ->
 	    filename:join(os:getenv("HOME"), ".wings");
 	{win32,_} ->
-            win32_new_pref()
+	    win32_new_pref()
     end.
 
 win32_new_pref() ->
@@ -241,16 +250,15 @@ win32_new_pref() ->
 
 win32_new_pref_1(R, [FolderType|T]) ->
     case win32_special_folder(R, FolderType) of
-        none -> win32_pref_1(R, T);
-        Path ->
-            File = filename:join(Path, ?WIN32_PREFS),
-            filelib:ensure_dir(File),
-            File
+	none -> win32_pref_1(R, T);
+	Path ->
+	    File = filename:join(Path, ?WIN32_PREFS),
+	    filelib:ensure_dir(File),
+	    File
     end;
 win32_new_pref_1(_, []) ->
-    %% Desperate fallback for very old Window systems.
-    %% (No "My Documents" folder.)
-    filename:join(code:lib_dir(wings), ?WIN32_PREFS).
+    %% Desperate fallback... (no standard folders found - should not happen).
+    filename:join(wings_util:lib_dir(wings), ?WIN32_PREFS).
 
 %%%
 %%% Utilities.
@@ -258,8 +266,8 @@ win32_new_pref_1(_, []) ->
 
 try_locations([D|Ds], File) ->
     case try_location(D, File) of
-        none -> try_locations(Ds, File);
-        Name -> Name
+	none -> try_locations(Ds, File);
+	Name -> Name
     end;
 try_locations([], _) -> none.
 
@@ -337,6 +345,8 @@ defaults() ->
     [{background_color,{0.8,0.8,0.8}},
      {info_color,{0.0,0.0,0.0}},
      {info_background_color,{0.8,0.8,0.8,0.5}},
+     {info_enhanced_text,false},
+     {info_text_on_hilite,false},
      {grid_color,{0.7,0.7,0.7}},
      {edge_color,{0.0,0.0,0.0}},
      {hard_edge_color,{0.0,0.5,0.0}},
@@ -354,8 +364,10 @@ defaults() ->
      {selected_vertex_size,5.0},
      {edge_width,2.0},
      {selected_edge_width,2.0},
+     {hard_edge_width,2.0},
      {show_axis_letters,true},
-     {constrain_axes,false},
+     {constrain_axes,true},
+     {mini_axis,true},
      {force_show_along_grid,false},
      {force_ortho_along_axis,false},
      {vertex_hilite,true},
@@ -370,8 +382,11 @@ defaults() ->
      {active_vector_width,2.0},
      {active_vector_color,{0.0,0.0,0.65}},
      {normal_vector_size,0.3},
+     {normal_vector_width,2.0},
      {normal_vector_color,{0.0,0.0,0.65}},
      {smart_highlighting,false},
+     {material_default,{1.0,1.0,1.0}},
+     {material_hole,{0.0,0.0,0.9}},
 
      {selection_style,solid},
      {hide_sel_while_dragging,false},
@@ -382,9 +397,8 @@ defaults() ->
      %% Compatibility preferences.
      {text_display_lists,true},
      {dummy_axis_letter,false},
-     {jumpy_camera,case {os:type(),os:version()} of
-		       {{unix,darwin},{Maj,Min,_}}
-		       when Maj >= 7, Min >= 3 -> true;
+     {jumpy_camera,case os:type() of
+		       {unix,darwin} -> true;
 		       _ -> false
 		   end},
      {polygon_offset_f,1.0},
@@ -396,6 +410,57 @@ defaults() ->
      {use_temp_sel,false},
      {use_super_temp_sel,false},
      {use_mirror_for_sels,true},
+     {clip_plane_color,{0.8,0.3,0.0}},
+     {clip_plane_size,1.5},
+
+     %% Constraints preferences.
+     {con_dist_alt,10.0},
+     {con_dist_shift,1.0},
+     {con_dist_ctrl,0.10},
+     {con_dist_ctrl_shift,0.010},
+     {con_dist_ctrl_alt,0.0010},
+     {con_dist_shift_alt,0.250},
+     {con_dist_ctrl_shift_alt,0.0250},
+
+     {con_dist_a_alt,1/64},
+     {con_dist_a_shift,1/4},
+     {con_dist_a_ctrl,1/8},
+     {con_dist_a_ctrl_shift,1/32},
+     {con_dist_a_ctrl_alt,5/8},
+     {con_dist_a_shift_alt,1/16},
+     {con_dist_a_ctrl_shift_alt,3/64},
+
+     {con_rot_alt,180.0},
+     {con_rot_shift,15.0},
+     {con_rot_ctrl,1.0},
+     {con_rot_ctrl_shift,0.10},
+     {con_rot_ctrl_alt,22.50},
+     {con_rot_shift_alt,135.0},
+     {con_rot_ctrl_shift_alt,144.0},
+
+     {con_scale_alt,10.0},
+     {con_scale_shift,1.0},
+     {con_scale_ctrl,0.10},
+     {con_scale_ctrl_shift,0.010},
+     {con_scale_ctrl_alt,0.0010},
+     {con_scale_shift_alt,0.250},
+     {con_scale_ctrl_shift_alt,0.0250},
+
+     {con_alternate,false},
+     {con_dist_set,con_dist_},
+
+     {con_dist_default,alt},
+     {con_rot_default,alt},
+     {con_scale_default,alt},
+
+     %% Drag preferences.
+     {drag_resets,false},
+     {drag_custom,false},
+     {drag_cam_dist_abs,true},
+     {drag_speed_abs,5.0},
+     {drag_cam_dist_relative,true},
+     {drag_speed_relative,5.0},
+     {drag_speed_rotate,5.0},
 
      %% Proxy preferences.
      {proxy_shaded_edge_style,some},
@@ -414,8 +479,11 @@ defaults() ->
      {title_passive_color,{0.325,0.4,0.325,1.0}},
      {title_text_color,{1.0,1.0,1.0}},
      {no_progress_bar,false},
-     {interface_icons,classic},
+     {interface_icons,bluecube},
      {objects_in_outliner,true},
+     {aa_edges,false},
+     {extended_toolbar,true},
+
      %% Console
      {console_width,80},
      {console_height,12},
@@ -488,6 +556,8 @@ not_bad(right_click_sel_in_ss, _) -> false;
 not_bad(right_click_sel_in_geom, _) -> false;
 not_bad(wire_edge_color, _) -> false;
 not_bad(show_wire_backfaces, _) -> false;	%Now a window property.
+not_bad(smoothed_preview_cage, _) -> false;
+not_bad(smoothed_preview_edges, _) -> false;
 
 %% Crashes have occurred.
 not_bad(last_axis, Val) -> is_wings_vector(Val);
