@@ -9,7 +9,7 @@
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: wings_render.erl,v 1.15 2006/07/27 02:22:24 antoneos Exp $
+%%     $Id: wings_render.erl 235 2008-01-24 08:30:08Z optigon $
 %%
 
 -module(wings_render).
@@ -22,25 +22,10 @@
 init() ->
     case wings_gl:support_shaders() of
 	true ->
-	    try 
-		Sh = wings_gl:compile(vertex, light_shader_src()),
-		Prog = wings_gl:link_prog([Sh]),
- 		gl:useProgram(Prog),
-		wings_pref:set_default(hl_lightpos,  {3.0,10.0,1.0}),
-		wings_pref:set_default(hl_skycol,    {0.95,0.95,0.90}),
-		wings_pref:set_default(hl_groundcol, {0.026,0.024,0.021}),
-		wings_gl:set_uloc(Prog, "LightPosition", 
-				  wings_pref:get_value(hl_lightpos)),
-		wings_gl:set_uloc(Prog, "SkyColor",
-				  wings_pref:get_value(hl_skycol)),
-		wings_gl:set_uloc(Prog, "GroundColor",
-				  wings_pref:get_value(hl_groundcol)),
-		?CHECK_ERROR(),
- 		gl:useProgram(0),
-		put(light_shader, Prog),
-		io:format("Using GPU shaders.\n"),
+	    try
+		wings_shaders:init()
+	    catch _:_Err ->
 		ok
-	    catch _:_Err -> ok
 	    end;
 	false ->
 	    ok
@@ -62,8 +47,11 @@ render(#st{selmode=Mode}=St) ->
     gl:enable(?GL_CULL_FACE),
     wings_view:load_matrices(true),
     ground_and_axes(),
+    mini_axis_icon(),
     show_saved_bb(St),
+    user_clipping_planes(on),
     render_objects(Mode),
+    user_clipping_planes(off),
     axis_letters(),
     gl:lineWidth(1),
     wings_io:ortho_setup(),
@@ -72,8 +60,6 @@ render(#st{selmode=Mode}=St) ->
     gl:rectf(W-0.5, 0.5, 0.5, H-0.5),
     gl:popAttrib().
 
-
-%%%
 polygonOffset(M) ->
     case get(polygon_offset) of
 	undefined ->
@@ -81,9 +67,10 @@ polygonOffset(M) ->
 	    R = wings_pref:get_value(polygon_offset_r,1.0),
 	    put(polygon_offset, {F,R}),
 	    gl:polygonOffset(M*F, M*R);
-	{F,R} -> 
+	{F,R} ->
 	    gl:polygonOffset(M*F, M*R)
     end.
+
 %%%
 %%% Internal functions follow.
 %%%
@@ -107,7 +94,7 @@ render_work_objects([D|Dls], Mode) ->
     render_object(D, Mode, true, false),
     render_work_objects(Dls, Mode);
 render_work_objects([], _) -> ok.
-    
+
 render_object(#dlo{drag={matrix,_,_,Matrix}}=D, Mode, Work, RT) ->
     gl:pushMatrix(),
     gl:multMatrixf(Matrix),
@@ -153,7 +140,6 @@ render_plain(#dlo{work=Faces,edges=Edges,open=Open,
 	    polygonOffset(2),
 	    gl:shadeModel(?GL_SMOOTH),
 	    enable_lighting(),
-	    
 	    case Open of
 		false ->
 		    wings_dl:call(Faces);
@@ -177,6 +163,16 @@ render_plain(#dlo{work=Faces,edges=Edges,open=Open,
 		{_,EdgeColor} ->
 		    gl:color3fv(EdgeColor)
 	    end,
+	    case wings_pref:get_value(aa_edges) of
+		true ->
+		    gl:enable(?GL_LINE_SMOOTH),
+		    gl:enable(?GL_BLEND),
+		    gl:blendFunc(?GL_SRC_ALPHA, ?GL_ONE_MINUS_SRC_ALPHA),
+		    gl:hint(?GL_LINE_SMOOTH_HINT, ?GL_NICEST),
+		    ok;
+		false ->
+		    ok
+	    end,
 	    gl:lineWidth(edge_width(SelMode)),
 	    gl:polygonMode(?GL_FRONT_AND_BACK, ?GL_LINE),
 	    gl:enable(?GL_POLYGON_OFFSET_LINE),
@@ -296,7 +292,7 @@ draw_sel(#dlo{orig_sel=OrigSel,sel=SelDlist}) ->
     gl:polygonOffset(1, 1),
     gl:polygonMode(?GL_FRONT_AND_BACK, ?GL_FILL),
     case OrigSel =/= none orelse wings_pref:get_value(selection_style) =:= solid of
-	true ->					%Solid selection style.
+	true -> 				%Solid selection style.
 	    wings_dl:call(SelDlist);
 	false ->				%Stippled selection style.
 	    gl:enable(?GL_POLYGON_STIPPLE),
@@ -344,18 +340,20 @@ draw_orig_sel_1(_, DlistSel) ->
 
 draw_hard_edges(#dlo{hard=none}, _) -> ok;
 draw_hard_edges(#dlo{hard=Hard}, SelMode) ->
-    gl:lineWidth(edge_width(SelMode)),
+    gl:lineWidth(hard_edge_width(SelMode)),
     gl:color3fv(wings_pref:get_value(hard_edge_color)),
     wings_dl:call(Hard).
 
 draw_normals(#dlo{normals=none}) -> ok;
 draw_normals(#dlo{normals=Ns}) ->
     gl:color3f(0, 0, 1),
-    gl:lineWidth(2),
+    gl:lineWidth(wings_pref:get_value(normal_vector_width)),
     wings_dl:call(Ns).
 
 edge_width(edge) -> wings_pref:get_value(edge_width);
 edge_width(_) -> 1.
+hard_edge_width(edge) -> wings_pref:get_value(hard_edge_width);
+hard_edge_width(_) -> wings_pref:get_value(hard_edge_width) -1.
 
 ground_and_axes() ->
     Axes = wings_wm:get_prop(show_axes),
@@ -444,7 +442,7 @@ axis_letters() ->
 	    end,
 	    axis_letter_1(1, Yon, axisx, x_color, Info),
 	    axis_letter_1(2, Yon, axisy, y_color, Info),
- 	    axis_letter_1(3, Yon, axisz, z_color, Info)
+	    axis_letter_1(3, Yon, axisz, z_color, Info)
     end.
 
 axis_letter_1(I, Yon, Char, Color0, {Start,{Ox,Oy,_,Ow},MM,PM,Viewport}) ->
@@ -579,46 +577,169 @@ show_saved_bb(#st{bb=[{X1,Y1,Z1},{X2,Y2,Z2}]}) ->
 show_saved_bb(_) -> ok.
 
 enable_lighting() ->
-    Prog = get(light_shader),
-    UseProg = 
-	(Prog /= undefined) andalso (not wings_pref:get_value(scene_lights)) 
-        andalso 2 == wings_pref:get_value(number_of_lights),
+    Progs = get(light_shaders),
+    NumLights = wings_pref:get_value(number_of_lights),
+    NumShaders = wings_pref:get_value(number_of_shaders),
+    UseProg = (Progs /= undefined) andalso
+	      (not wings_pref:get_value(scene_lights)) andalso
+	      (NumLights == 2),
     case UseProg of
-	false -> 
+	false ->
 	    gl:enable(?GL_LIGHTING);
 	true ->
+	    {Prog,_Name} = element(NumShaders, Progs),
 	    gl:color4ub(255,255,255,255), %% Reset color needed by crappy drivers.
 	    %% We put it here and not in apply_material because we can't use some
 	    %% optimizations (i.e. reuse display lists) when drawing selected objects
 	    gl:useProgram(Prog)
     end.
 
-
 disable_lighting() ->
     gl:disable(?GL_LIGHTING),
-    case get(light_shader) /= undefined of
- 	true -> gl:useProgram(0);
+    case get(light_shaders) /= undefined of
+	true -> gl:useProgram(0);
 	false -> ok
     end.
 
+mini_axis_icon() ->
+    case wings_pref:get_value(mini_axis) of 
+    false -> ok;
+    true ->
+      gl:pushAttrib(?GL_ALL_ATTRIB_BITS),
+      {W,H} = wings_wm:win_size(),
+      Matrix1 = list_to_tuple(gl:getDoublev(?GL_MODELVIEW_MATRIX)),
+      Matrix2 = setelement(13, Matrix1, -W/H+0.11),
+      Matrix3 = setelement(14, Matrix2, -1.0+0.11),
+      Matrix4 = setelement(15, Matrix3, 0.0),
+      gl:matrixMode(?GL_PROJECTION),
+      gl:pushMatrix(),
+      gl:loadIdentity(),
+      gl:ortho(-W/H, W/H, -1, 1, 0.00001, 10000000.0),
+      gl:matrixMode(?GL_MODELVIEW),
+      gl:pushMatrix(),
+      gl:loadIdentity(),
+      gl:loadMatrixd(Matrix4),
+      draw_mini_axis(),
+      gl:popMatrix(),
+      gl:matrixMode(?GL_PROJECTION),
+      gl:popMatrix(),
+      gl:matrixMode(?GL_MODELVIEW),
+      gl:popAttrib()
+	end.
+
+draw_mini_axis() ->
+    {PA,PB} = {0.08,0.01},
+    X = wings_pref:get_value(x_color),
+    Y = wings_pref:get_value(y_color),
+    Z = wings_pref:get_value(z_color),
+    gl:'begin'(?GL_LINES),
+    %% X Axis
+    gl:color3fv(X),
+    gl:vertex3f(0,0,0),
+    gl:vertex3f(0.1,0.0,0.0),
+    %% Y Axis
+    gl:color3fv(Y),
+    gl:vertex3f(0,0,0),
+    gl:vertex3f(0.0,0.1,0.0),
+    %% Z Axis
+    gl:color3fv(Z),
+    gl:vertex3f(0,0,0),
+    gl:vertex3f(0.0,0.0,0.1),
+    View = wings_view:current(),
+    case View#view.along_axis of
+	none ->
+	    %% X Arrows
+	    gl:color3fv(X),
+	    gl:vertex3f(PA,0.0,-PB),
+	    gl:vertex3f(0.1,0.0,0.0),
+	    gl:vertex3f(PA,0.0,PB),
+	    gl:vertex3f(0.1,0.0,0.0),
+	    %% Y Arrows
+	    gl:color3fv(Y),
+	    gl:vertex3f(-PB,PA,0.0),
+	    gl:vertex3f(0.0,0.1,0.0),
+	    gl:vertex3f(PB,PA,0.0),
+	    gl:vertex3f(0.0,0.1,0.0),
+	    %% Z Arrows
+	    gl:color3fv(Z),
+	    gl:vertex3f(-PB,0.0,PA),
+	    gl:vertex3f(0.0,0.0,0.1),
+	    gl:vertex3f(PB,0.0,PA),
+	    gl:vertex3f(0.0,0.0,0.1);
+	x ->
+	    %% Y Arrows
+	    gl:color3fv(Y),
+	    gl:vertex3f(0.0,PA,-PB),
+	    gl:vertex3f(0.0,0.1,0.0),
+	    gl:vertex3f(0.0,PA,PB),
+	    gl:vertex3f(0.0,0.1,0.0),
+	    %% Z Arrows
+	    gl:color3fv(Z),
+	    gl:vertex3f(0.0,-PB,PA),
+	    gl:vertex3f(0.0,0.0,0.1),
+	    gl:vertex3f(0.0,PB,PA),
+	    gl:vertex3f(0.0,0.0,0.1);
+	y ->
+	    %% X Arrows
+	    gl:color3fv(X),
+	    gl:vertex3f(PA,0.0,-PB),
+	    gl:vertex3f(0.1,0.0,0.0),
+	    gl:vertex3f(PA,0.0,PB),
+	    gl:vertex3f(0.1,0.0,0.0),
+	    %% Z Arrows
+	    gl:color3fv(Z),
+	    gl:vertex3f(-PB,0.0,PA),
+	    gl:vertex3f(0.0,0.0,0.1),
+	    gl:vertex3f(PB,0.0,PA),
+	    gl:vertex3f(0.0,0.0,0.1);
+	z ->
+	    %% X Arrows
+	    gl:color3fv(X),
+	    gl:vertex3f(PA,-PB,0.0),
+	    gl:vertex3f(0.1,0.0,0.0),
+	    gl:vertex3f(PA,PB,0.0),
+	    gl:vertex3f(0.1,0.0,0.0),
+	    %% Y Arrows
+	    gl:color3fv(Y),
+	    gl:vertex3f(-PB,PA,0.0),
+	    gl:vertex3f(0.0,0.1,0.0),
+	    gl:vertex3f(PB,PA,0.0),
+	    gl:vertex3f(0.0,0.1,0.0)
+    end,
+    gl:'end'().
+
+user_clipping_planes(on) ->
+    case wings_wm:get_prop(clip_plane) of
+	true ->
+	    {_Position,Direction} = wings_pref:get_value(last_axis),
+	    Expand = fun(V) -> {X,Y,Z}=V, [X,Y,Z,0.0] end,
+	    Plane0 = Expand(Direction),
+	    draw_clip_disk(Direction, Expand),
+	    gl:clipPlane(?GL_CLIP_PLANE0, Plane0),
+	    gl:enable(?GL_CLIP_PLANE0),
+	    gl:disable(?GL_CULL_FACE);
+	false ->
+	    ok
+    end;
+user_clipping_planes(off) ->
+    gl:disable(?GL_CLIP_PLANE0),
+    gl:enable(?GL_CULL_FACE).
+
+draw_clip_disk(Direction, Expand) ->
+    NZ = e3d_vec:norm(Direction),
+    NX = e3d_vec:norm(e3d_vec:cross(NZ,{0.0,0.0,1.0})),
+    NY = e3d_vec:cross(NX,NZ),
+    Nx = Expand(NX),
+    Ny = Expand(NY),
+    Nz = Expand(NZ),
+    Nw = [0.0,0.0,0.0,1.0],
+    M = lists:append([Nx,Ny,Nz,Nw]),
+    Obj = glu:newQuadric(),
+    glu:quadricDrawStyle(Obj, ?GLU_SILHOUETTE),
+    gl:pushMatrix(),
+    gl:multMatrixd(M),
+    gl:color3fv(wings_pref:get_value(clip_plane_color)),
+    glu:disk(Obj, 0.0, wings_pref:get_value(clip_plane_size), 35, 1),
+    gl:popMatrix(),
+    glu:deleteQuadric(Obj).
 
-light_shader_src() ->
-    <<"							       
-       uniform vec3 LightPosition;				       
-       uniform vec3 SkyColor;					       
-       uniform vec3 GroundColor;				       
-       							       
-       void main() 						       
-       {							       
-           vec3 ecPosition = vec3(gl_ModelViewMatrix * gl_Vertex);    
-           vec3 tnorm      = normalize(gl_NormalMatrix * gl_Normal);  
-           vec3 lightVec   = normalize(LightPosition - ecPosition);   
-           float costheta  = dot(tnorm, lightVec);
-           float a         = 0.5 + 0.5 * costheta;
-                             // ATI needs this for vcolors to work
-           vec4 color      = gl_FrontMaterial.diffuse * gl_Color; 
-           gl_FrontColor   = color * vec4(mix(GroundColor, SkyColor, a), 1.0);
-           gl_TexCoord[0]  = gl_MultiTexCoord0;
-           gl_Position     = ftransform();
-       }							       
-       ">>.
