@@ -3,12 +3,12 @@
 %%
 %%     This module implements most of the commands in the View menu.
 %%
-%%  Copyright (c) 2001-2004 Bjorn Gustavsson
+%%  Copyright (c) 2001-2008 Bjorn Gustavsson
 %%
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: wings_view.erl,v 1.170 2006/06/27 16:58:56 giniu Exp $
+%%     $Id: wings_view.erl 296 2008-03-24 17:23:50Z optigon $
 %%
 
 -module(wings_view).
@@ -35,6 +35,8 @@ menu(#st{views={CurrentView,Views}}=St) ->
      separator,
      {?__(5,"Workmode"),workmode,?__(6,"Toggle flat/smooth shading"),
       crossmark(workmode)},
+     {?__(61,"Clipping Planes"),clip_plane,?__(62,"Toggle extra clipping planes (use default axis)"),
+      crossmark(clip_plane)},
      separator,
      {?__(7,"Wireframe"),wireframe,?__(8,"Display selected objects as a wireframe (same for all objects if nothing is selected)")},
      {?__(9,"Shade"),shade,?__(10,"Display selected objects as shaded (same for all objects if nothing is selected)")},
@@ -56,10 +58,11 @@ menu(#st{views={CurrentView,Views}}=St) ->
      {?__(25,"Reset View"),reset,?__(26,"Reset view to the default position")},
      {?__(27,"Aim"),aim,?__(28,"Aim the camera at the selected element")},
      {?__(29,"Frame"),frame,?__(30,"Dolly to show all selected elements (or all objects if nothing is selected)")},
+	 {?__(65,"Frame Disregards Mirror"),frame_mode,crossmark(frame_disregards_mirror)},
      {?__(31,"Orthographic View"),orthogonal_view,
       ?__(32,"Toggle between orthographic and perspective views"),
       crossmark(orthogonal_view)},
-     {?__(33,"Saved Views: ")++integer_to_list(size(Views)),
+     {?__(33,"Saved Views: ")++integer_to_list(tuple_size(Views)),
       {views,views_submenu(CurrentView, Views)}},
      separator,
      {?__(34,"Camera Settings..."),camera_settings,?__(35,"Set field of view, and near and far clipping planes")},
@@ -70,6 +73,11 @@ menu(#st{views={CurrentView,Views}}=St) ->
      {one_of(L == 1, ?__(38,"Two Lights"),?__(39,"One Light")),toggle_lights,
       one_of(L == 1, ?__(40,"Use two work lights"),
 	     ?__(41,"Use one work light"))},
+%% If there should be shaders, there should be a better mechanism for choosing shaders,
+%% and preferably a way to turn them of since they have limited use in a modeller.
+%%    -- Bjorn G
+%%     {?__(63,"Activate Next Shader"),cycle_shaders,
+%%      ?__(64,"Cycle through all vertex/fragment shaders")},
      separator,
      {?__(42,"Show Colors"),show_colors,
       ?__(43,"Show vertex colors on objects in \"vertex\" mode"),
@@ -129,7 +137,7 @@ wireframe_crossmark(#st{sel=Sel0}) ->
 views_submenu(CurrentView, Views) ->
     {_,_,_,H} = wings_wm:viewport(desktop),
     Lines = wings_util:max((H div ?LINE_HEIGHT) - 3, 4),
-    S = size(Views),
+    S = tuple_size(Views),
     C = if S > 0 -> view_index(CurrentView, S); true -> 0 end,
     [{?__(1,"Next"),next,views_submenu_help(CurrentView, Views, next)},
      {?__(2,"Current"),current,views_submenu_help(CurrentView, Views, current)},
@@ -146,21 +154,21 @@ views_submenu(CurrentView, Views) ->
 views_submenu_help(_CurrentView, {}, _Action) ->
     ?__(1,"No saved views!");
 views_submenu_help(CurrentView, Views, Action) ->
-    S = size(Views),
+    S = tuple_size(Views),
     case Action of
-	next -> 
+	next ->
 	    N = view_index(CurrentView+1, S),
 	    {_,Legend} = element(N, Views),
 	    ?__(2,"Jump to \"")++Legend++"\"["++integer_to_list(N)++"]";
-	current -> 
+	current ->
 	    C = view_index(CurrentView, S),
 	    {_,Legend} = element(C, Views),
 	    ?__(2,"Jump to \"")++Legend++"\"["++integer_to_list(C)++"]";
-	prev -> 
+	prev ->
 	    P = view_index(CurrentView-1, S),
 	    {_,Legend} = element(P, Views),
 	    ?__(2,"Jump to \"")++Legend++"\"["++integer_to_list(P)++"]";
-	rename -> 
+	rename ->
 	    C = view_index(CurrentView, S),
 	    {_,Legend} = element(C, Views),
 	    ?__(5,"Rename \"")++Legend++"\"["++integer_to_list(C)++"]";
@@ -172,7 +180,7 @@ views_submenu_help(CurrentView, Views, Action) ->
 	    ?__(11,"Delete all saved views")
     end.
 
-view_index(I, N) when integer(I), integer(N), N > 0 ->
+view_index(I, N) when is_integer(I), is_integer(N), N > 0 ->
     J = (I-1) rem N,
     if J < 0 -> J + 1 + N;
        true -> J + 1
@@ -181,7 +189,7 @@ view_index(I, N) when integer(I), integer(N), N > 0 ->
 views_jumpmenu(_CurrentView, {}, _Lines) ->
     {?__(1,"Jump"),current,?__(2,"No saved views!")};
 views_jumpmenu(CurrentView, Views, Lines) ->
-    S = size(Views),
+    S = tuple_size(Views),
     P = view_index(CurrentView-1, S),
     C = view_index(CurrentView, S),
     N = view_index(CurrentView+1, S),
@@ -201,7 +209,7 @@ views_jumpmenu(CurrentView, Views, Lines) ->
 views_movemenu(_CurrentView, {}, _Lines) ->
     {?__(1,"Move Current"),current,?__(2,"No saved views!")};
 views_movemenu(CurrentView, Views, Lines) ->
-    S = size(Views),
+    S = tuple_size(Views),
     P = view_index(CurrentView-1, S),
     C = view_index(CurrentView, S),
     N = view_index(CurrentView+1, S),
@@ -223,11 +231,11 @@ views_movemenu(CurrentView, Views, Lines) ->
 
 %% Build a list of F(I) values for all view_index(I, S) starting from C
 %% half of them after C and half before, no more than Lines If the
-%% list has to be limited by Lines - insert an element 'separator' 
+%% list has to be limited by Lines - insert an element 'separator'
 %% between the after and before elements, otherwise all view
 %% indexes from 1 upto S will be represented in the result with
 %% C last in the list.
-%% 
+%%
 %% Like this when Lines == 4, S > 4 ->
 %%   [F(view_index(C+1, S)), F(view_index(C+2, S)), separator,
 %%    F(view_index(C-1, S)), F(view_index(C, S))].
@@ -334,6 +342,10 @@ command(aim, St) ->
 command(frame, St) ->
     frame(St),
     St;
+command(frame_mode, St) ->
+    Bool = wings_pref:get_value(frame_disregards_mirror),
+    wings_pref:set_value(frame_disregards_mirror, not Bool),
+	St;
 command({views,Views}, St) ->
     views(Views, St);
 command({along,Axis}, St) ->
@@ -352,6 +364,9 @@ command(align_to_selection, St) ->
 command(toggle_lights, St) ->
     toggle_lights(),
     St;
+%% command(cycle_shaders, St) ->
+%%     cycle_shaders(),
+%%     St;
 command(camera_settings, St) ->
     camera(),
     St;
@@ -460,15 +475,15 @@ camera() ->
 	    {60,60} -> {60,60};
 	    _ -> custom
 	end,
-    Props = 
-	camera_propconv(from_fov, 
+    Props =
+	camera_propconv(from_fov,
 			[{negative_height,NegH},{negative_width,NegW},
 			 {fov,Fov0}]),
     LensType = pget(lens_type, Props),
     LensLength = pget(lens_length, Props),
     Zoom = pget(zoom, Props),
     ZoomSlider = pget(zoom_slider, Props),
-    FovHook = 
+    FovHook =
 	fun (update, {Var,_I,Val,Sto}) ->
 		{store,camera_update_1(Var, Val, Sto)};
 	    (_, _) -> void
@@ -557,7 +572,7 @@ camera() ->
 	       end}]},
 	    {label,?__(13,"x Zoom")}]}],
 	 [{title,?__(14,"Lens")},{minimized,true}]},
-    Qs = 
+    Qs =
 	[LensFrame,
 	 {hframe,
 	  [{vframe,[{label,?__(15,"Field of View")},
@@ -586,13 +601,13 @@ camera() ->
 
 camera_update_1(Var, Val, Sto) ->
     Props = gbget(lists:delete(Var, [fov,negative_format,
-				     negative_height,negative_width]), 
+				     negative_height,negative_width]),
 		  Sto),
     gbupdate(camera_propconv(from_fov, [{Var,Val}|Props]),
 	     gbupdate(Var, Val, Sto)).
 
 camera_update_2(Var, Val, Sto) ->
-    [_|Props0] = Props1 = 
+    [_|Props0] = Props1 =
 	gbget([lens_length,
 	       negative_format,negative_height,negative_width], Sto),
     Props2 = camera_propconv(fov, [{Var,Val}|Props1])++Props0,
@@ -602,7 +617,7 @@ camera_update_2(Var, Val, Sto) ->
 camera_propconv(from_fov, Props) ->
     {NegH,NegW} = NegF = camera_propconv_negative_format(Props),
     Fov = pget(fov, Props),
-    LensLength = 
+    LensLength =
 	case catch 0.5*NegH/math:tan(Fov*math:pi()/360) of
 	    {'EXIT',_} -> 0.0;
 	    L when is_float(L) -> L
@@ -611,9 +626,9 @@ camera_propconv(from_fov, Props) ->
 	camera_lens_type(NegF, LensLength),
     Zoom =
 	LensLength / math:sqrt(NegH*NegH + NegW*NegW),
-    ZoomSlider = 
+    ZoomSlider =
 	case catch math:log(Zoom) / math:log(2) of
-	    {'EXIT',_} -> 
+	    {'EXIT',_} ->
 		{Z,_} = ?RANGE_ZOOM_SLIDER,
 		Z;
 	    Z when is_float(Z) -> Z
@@ -624,14 +639,14 @@ camera_propconv(from_fov, Props) ->
      {zoom_slider,wings_util:limit(ZoomSlider, ?RANGE_ZOOM_SLIDER)}];
 camera_propconv(fov, Props) ->
     {NegH,NegW} = NegF = camera_propconv_negative_format(Props),
-    Zoom = 
+    Zoom =
 	case pget(zoom_slider, Props) of
-	    undefined -> 
+	    undefined ->
 		pget(zoom, Props);
 	    ZoomSlider ->
 		math:pow(2, ZoomSlider)
 	end,
-    LensLength = 
+    LensLength =
 	case Zoom of
 	    undefined ->
 		case pget(lens_type, Props) of
@@ -639,13 +654,13 @@ camera_propconv(fov, Props) ->
 			pget(lens_length, Props);
 		    undefined ->
 			pget(lens_length, Props);
-		    LensType -> 
+		    LensType ->
 			camera_lens_length(NegF, LensType)
 		end;
 	    _ ->
 		Zoom * math:sqrt(NegH*NegH + NegW*NegW)
 	end,
-    Fov = 
+    Fov =
 	if is_float(LensLength) ->
 		case catch 360*math:atan(0.5*NegH/LensLength)/math:pi() of
 		    {'EXIT',_} -> 180.0;
@@ -656,9 +671,9 @@ camera_propconv(fov, Props) ->
 
 camera_propconv_negative_format(Props) ->
     case pget(negative_format, Props) of
-	{_,_} = NegativeFormat -> 
+	{_,_} = NegativeFormat ->
 	    NegativeFormat;
-	_ -> 
+	_ ->
 	    {pget(negative_height, Props),
 	     pget(negative_width, Props)}
     end.
@@ -685,19 +700,19 @@ camera_lens_type(_, _) -> custom.
 
 camera_lens_length({24,36}, LensType) ->
     float(case LensType of
-	      wide_angle          -> 24;
+	      wide_angle	  -> 24;
 	      moderate_wide_angle -> 35;
-	      standard            -> 50;
-	      short_tele          -> 85;
-	      tele                -> 135
+	      standard		  -> 50;
+	      short_tele	  -> 85;
+	      tele		  -> 135
 	  end);
 camera_lens_length({60,60}, LensType) ->
     float(case LensType of
-	      wide_angle          -> 40;
+	      wide_angle	  -> 40;
 	      moderate_wide_angle -> 60;
-	      standard            -> 80;
-	      short_tele          -> 150;
-	      tele                -> 250
+	      standard		  -> 80;
+	      short_tele	  -> 150;
+	      tele		  -> 250
 	  end);
 camera_lens_length(_, _) -> undefined.
 
@@ -723,8 +738,8 @@ pget(Key, Props) -> proplists:get_value(Key, Props).
 %%%
 
 -record(tim,
-	{timer,					%Current timer.
-	 delay,					%Current delay.
+	{timer, 				%Current timer.
+	 delay, 				%Current delay.
 	 st					%St record.
 	 }).
 
@@ -735,7 +750,7 @@ auto_rotate(St) ->
     Active = wings_wm:this(),
     wings_wm:callback(fun() -> wings_u:menu_restriction(Active, []) end),
     {seq,push,set_auto_rotate_timer(Tim)}.
-    
+
 auto_rotate_event(Event, #tim{timer=Timer,st=St}=Tim) ->
     case wings_camera:event(Event, St) of
 	next -> auto_rotate_event_1(Event, Tim);
@@ -819,20 +834,20 @@ set_current(View) ->
 init() ->
     wings_pref:set_default(show_edges, true),
     wings_pref:set_default(number_of_lights, 1),
+    wings_pref:set_default(number_of_shaders, 1),
     wings_pref:set_default(show_normals, false),
     wings_pref:set_default(show_bb, true),
     wings_pref:set_default(show_colors, true),
     wings_pref:set_default(show_materials, true),
     wings_pref:set_default(show_textures, true),
+    wings_pref:set_default(frame_disregards_mirror, false),
 
-    wings_pref:set_default(scene_lights, false),
-
-    wings_pref:set_default(smoothed_preview_cage, false),
-    wings_pref:set_default(smoothed_preview_edges, false).
+    wings_pref:set_default(scene_lights, false).
 
 initial_properties() ->
     [{workmode,true},
      {orthogonal_view,false},
+     {clip_plane,false},
      {show_axes,true},
      {show_groundplane,true},
      {wireframed_objects,gb_sets:empty()},
@@ -873,10 +888,10 @@ load_matrices(IncludeLights) ->
 projection() ->
     {W,H} = wings_wm:win_size(),
     Aspect = W/H,
-    #view{distance=D,fov=Fov,hither=Hither,yon=Yon,along_axis=AA} = 
+    #view{distance=D,fov=Fov,hither=Hither,yon=Yon,along_axis=AA} =
 	current(),
-    Ortho = wings_wm:get_prop(orthogonal_view) 
-	orelse ((AA =/= none) andalso 
+    Ortho = wings_wm:get_prop(orthogonal_view)
+	orelse ((AA =/= none) andalso
 		wings_pref:get_value(force_ortho_along_axis)),
     case Ortho of
 	false ->
@@ -929,8 +944,8 @@ aim(St) ->
     #view{distance=Dist0} = View = current(),
     Dist = case e3d_vec:dist(eye_point(), Origin0) of
 	       D when D < Dist0 -> D;
- 	       _Other -> Dist0
- 	   end,
+	       _Other -> Dist0
+	   end,
     set_current(View#view{origin=Origin,distance=Dist,pan_x=0.0,pan_y=0.0}).
 
 frame(#st{sel=[],shapes=Shs}) ->
@@ -943,7 +958,14 @@ frame(#st{sel=[],shapes=Shs}) ->
 	       end,
 	       none, gb_trees:values(Shs)),
     frame_1(BB);
-frame(St) -> frame_1(wings_sel:bounding_box(St)).
+frame(St0) ->
+    St = case wings_pref:get_value(frame_disregards_mirror) of
+	true ->
+      kill_mirror(St0);
+	false -> 
+	  St0
+	end,
+    frame_1(wings_sel:bounding_box(St)).
 
 frame_1(none) -> ok;
 frame_1([A,B]=BB) ->
@@ -954,15 +976,29 @@ frame_1([A,B]=BB) ->
     set_current(View#view{origin=e3d_vec:neg(C),
 			  distance=Dist,pan_x=0.0,pan_y=0.0}).
 
-views({save,[Legend]}, #st{views={_,{}}}=St) ->
-    St#st{views={1,{{current(),Legend}}}};
-views({save,[Legend]}, #st{views={CurrentView,Views}}=St) ->
-    J = view_index(CurrentView, size(Views)),
+kill_mirror(#st{shapes=Shs0}=St) ->
+    Shs = kill_mirror_1(gb_trees:values(Shs0), []),
+    St#st{shapes=Shs}.
+
+kill_mirror_1([#we{id=Id,mirror=none}=We|Wes], Acc) ->
+    kill_mirror_1(Wes, [{Id,We}|Acc]);
+kill_mirror_1([#we{id=Id}=We|Wes], Acc) ->
+    kill_mirror_1(Wes, [{Id,We#we{mirror=none}}|Acc]);
+kill_mirror_1([], Acc) ->
+    gb_trees:from_orddict(lists:reverse(Acc)).
+
+views({save,[Legend]}, #st{views={_,{}}}=St0) ->
+    St = St0#st{views={1,{{current(),Legend}}},saved=false},
+    wings_u:caption(St);
+views({save,[Legend]}, #st{views={CurrentView,Views}}=St0) ->
+    J = view_index(CurrentView, tuple_size(Views)),
     {L1,L2} = lists:split(J, tuple_to_list(Views)),
-    St#st{views={J+1,list_to_tuple(L1++[{current(),Legend}|L2])}};
+    St = St0#st{views={J+1,list_to_tuple(L1++[{current(),Legend}|L2])},
+		saved=false},
+    wings_u:caption(St);
 views({save,Ask}, #st{views={CurrentView,Views}}) when is_atom(Ask) ->
     View = current(),
-    S = size(Views),
+    S = tuple_size(Views),
     if S > 0 ->
 	    case element(view_index(CurrentView, S), Views) of
 		{View,_} ->
@@ -976,16 +1012,16 @@ views({save,Ask}, #st{views={CurrentView,Views}}) when is_atom(Ask) ->
 views(_, #st{views={_,{}}}) ->
     wings_u:message(?__(2,"No saved views"));
 views(next, #st{views={CurrentView,Views}}=St) ->
-    J = view_index(CurrentView + 1, size(Views)),
+    J = view_index(CurrentView + 1, tuple_size(Views)),
     {View,_} = element(J, Views),
     set_current(View),
     St#st{views={J,Views}};
 views(current, #st{views={CurrentView,Views}}) ->
-    {View,_} = element(view_index(CurrentView, size(Views)), Views),
+    {View,_} = element(view_index(CurrentView, tuple_size(Views)), Views),
     set_current(View),
     wings_wm:dirty();
 views(prev, #st{views={CurrentView,Views}}=St) ->
-    J = view_index(CurrentView - 1, size(Views)),
+    J = view_index(CurrentView - 1, tuple_size(Views)),
     {View,_} = element(J, Views),
     set_current(View),
     St#st{views={J,Views}};
@@ -994,7 +1030,7 @@ views({jump,J}, #st{views={CurrentView,Views}}=St) ->
 views({move,J}, #st{views={CurrentView,Views}}=St) ->
     views_move(J, St, CurrentView, Views);
 views(rename, #st{views={CurrentView,Views}}=St) ->
-    J = view_index(CurrentView, size(Views)),
+    J = view_index(CurrentView, tuple_size(Views)),
     {View,Legend} = element(J, Views),
     wings_ask:dialog(
       ?__(3,"Rename view"),
@@ -1005,7 +1041,7 @@ views(rename, #st{views={CurrentView,Views}}=St) ->
       end);
 views(delete, #st{views={CurrentView,Views}}=St) ->
     View = current(),
-    J = view_index(CurrentView, size(Views)),
+    J = view_index(CurrentView, tuple_size(Views)),
     case element(J, Views) of
 	{View,_} ->
 	    I = J - 1,
@@ -1018,13 +1054,13 @@ views(delete_all, St) ->
     This = wings_wm:this(),
     wings_u:yes_no(
       ?__(5,"Are you sure you want to delete all saved views?"),
-      fun() -> 
+      fun() ->
 	      wings_wm:send(This, {new_state,delete_all(St)}),
 	      ignore
       end).
 
 views_save_dialog(Ask, Options) ->
-    wings_ask:dialog(Ask, ?__(1,"Save view as"), 
+    wings_ask:dialog(Ask, ?__(1,"Save view as"),
 		     views_rename_qs(Options),
 		     fun(Opts) -> {view,{views,{save,Opts}}} end).
 
@@ -1032,7 +1068,7 @@ views_rename_qs([Legend]) ->
     [{hframe,[{label,?__(1,"Name")},{text,Legend}]}].
 
 views_jump(J, St, CurrentView, Views) ->
-    S = size(Views),
+    S = tuple_size(Views),
     case {view_index(J, S),view_index(CurrentView, S)} of
 	{J,J} ->
 	    {View,_} = element(J, Views),
@@ -1047,7 +1083,7 @@ views_jump(J, St, CurrentView, Views) ->
     end.
 
 views_move(J, St, CurrentView, Views) ->
-    S = size(Views),
+    S = tuple_size(Views),
     case {view_index(J, S),view_index(CurrentView, S)} of
 	{J,J} ->
 	    {View,_} = element(J, Views),
@@ -1074,6 +1110,21 @@ toggle_lights() ->
 	     end,
     wings_pref:set_value(number_of_lights, Lights).
 
+%% cycle_shaders() ->
+%%     case wings_gl:support_shaders() of
+%% 	true ->
+%% 	    NumShaders = wings_pref:get_value(number_of_shaders),
+%% 	    NumProgs = tuple_size(get(light_shaders)),
+%% 	    Shaders = case NumShaders of
+%% 			  NumProgs -> 1;
+%% 			  _ -> NumShaders+1
+%% 		      end,
+%% 	    wings_pref:set_value(number_of_lights, 2),
+%% 	    wings_pref:set_value(number_of_shaders, Shaders);
+%% 	false ->
+%% 	    toggle_lights()
+%%     end.
+
 along(x) -> along(x, -90.0, 0.0);
 along(y) -> along(y, 0.0, 90.0);
 along(z) -> along(z, 0.0, 0.0);
@@ -1189,9 +1240,9 @@ import_view([{yon,Yon}|As], View, Name) ->
     import_view(As, View#view{yon=Yon}, Name);
 import_view([{name,Name}|As], View, _) ->
     import_view(As, View, Name);
-import_view([], #view{azimuth=Az,elevation=El}=View, undefined) -> 
+import_view([], #view{azimuth=Az,elevation=El}=View, undefined) ->
     {View#view{along_axis=along(Az, El)},view_legend(View)};
-import_view([], #view{azimuth=Az,elevation=El}=View, Name) -> 
+import_view([], #view{azimuth=Az,elevation=El}=View, Name) ->
     {View#view{along_axis=along(Az, El)},Name}.
 
 %%%
@@ -1269,8 +1320,8 @@ add_dist_str(S) -> [$\s|S].
 pos_legend({X,Y,Z}) ->
     %% Sort the coordinates as greatest absolute value first.
     %% Remove the ones with absolute value lower then tan(45/2) times
-    %% the maximum. This should use the same style as N, NW, W type 
-    %% compass direction notation where each value has 
+    %% the maximum. This should use the same style as N, NW, W type
+    %% compass direction notation where each value has
     %% a precision of +-45/2 degrees.
     %% Return e.g "+X+Y", "-Z+X" depending on the sign of the remaining
     %% coordinates, largest first, as a deep charlist.
@@ -1278,7 +1329,7 @@ pos_legend({X,Y,Z}) ->
 	    lists:reverse(lists:sort(
 			    [{abs(X),$X,X},{abs(Y),$Y,Y},{abs(Z),$Z,Z}])),
     [[if Val < 0 -> $-;
-	 true    -> $+ end,Char] 
+	 true	 -> $+ end,Char]
      || {Abs,Char,Val} <- L, Abs >= Max*0.4142135624]. % tan(22.5 degrees)
 
 
