@@ -5,12 +5,12 @@
 %%     we records (winged-edged records, the central data structure
 %%     in Wings 3D).
 %%
-%%  Copyright (c) 2001-2005 Bjorn Gustavsson
+%%  Copyright (c) 2001-2007 Bjorn Gustavsson
 %%
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: wings_we.erl,v 1.110 2006/01/27 09:20:55 dgud Exp $
+%%     $Id: wings_we.erl 178 2008-01-01 10:11:00Z bjorng $
 %%
 
 -module(wings_we).
@@ -510,7 +510,6 @@ build_faces_1([{Face,[Edge|_]}|Fs], Acc) ->
     build_faces_1(Fs, [{Face,Edge}|Acc]);
 build_faces_1([], Acc) -> gb_trees:from_orddict(reverse(Acc)).
 
-fill_holes([]) ->  [];
 fill_holes(Es) ->
     G = digraph:new(),
     make_digraph(Es, G),
@@ -522,7 +521,7 @@ fill_holes(Es) ->
 make_hole_faces(G, [[V|_]|Cs], Acc) ->
     case digraph:get_cycle(G, V) of
 	[_|Vs] when length(Vs) >= 3 ->
-	    length(Vs) =:= length(ordsets:from_list(Vs)),
+	    true = length(Vs) =:= length(ordsets:from_list(Vs)),
 	    make_hole_faces(G, Cs, [{'_hole_',Vs}|Acc]);
 	_Other ->
 	    make_hole_faces(G, Cs, Acc)
