@@ -8,7 +8,7 @@
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: wings_wm.erl,v 1.154 2005/08/16 09:10:16 dgud Exp $
+%%     $Id: wings_wm.erl 217 2008-01-06 22:53:59Z bjorng $
 %%
 
 -module(wings_wm).
@@ -534,12 +534,13 @@ dispatch_matching(Filter) ->
 
 dispatch_event(#resize{w=W,h=H}) ->
     ?CHECK_ERROR(),
-    {SaveW,SaveH} =
-	case sdl_video:wm_isMaximized() of
-	    false -> {W,H};
-	    true ->  {W-8,H-10}
-	end,
-    wings_pref:set_value(window_size, {SaveW,SaveH}),
+    %% If the window has become maximized, we don't want
+    %% to save the window size, but will keep the previous
+    %% size.
+    case sdl_video:wm_isMaximized() of
+	false -> wings_pref:set_value(window_size, {W,H});
+	true -> ok
+    end,
     put(wm_top_size, {W,H}),
     init_opengl(),
     resize_windows(W, H),
